/* Created by build_access.py */
/* returns string of length 0 if invalid arg */
const char * dwarf_get_elf_relocname_ppc(unsigned long);
#ifndef R_PPC_NONE
#define R_PPC_NONE           0
#endif /* R_PPC_NONE */
#ifndef R_PPC_ADDR32
#define R_PPC_ADDR32         1
#endif /* R_PPC_ADDR32 */
#ifndef R_PPC_ADDR24
#define R_PPC_ADDR24         2
#endif /* R_PPC_ADDR24 */
#ifndef R_PPC_ADDR16
#define R_PPC_ADDR16         3
#endif /* R_PPC_ADDR16 */
#ifndef R_PPC_ADDR16_LO
#define R_PPC_ADDR16_LO      4
#endif /* R_PPC_ADDR16_LO */
#ifndef R_PPC_ADDR16_HI
#define R_PPC_ADDR16_HI      5
#endif /* R_PPC_ADDR16_HI */
#ifndef R_PPC_ADDR16_HA
#define R_PPC_ADDR16_HA      6
#endif /* R_PPC_ADDR16_HA */
#ifndef R_PPC_ADDR14
#define R_PPC_ADDR14         7
#endif /* R_PPC_ADDR14 */
#ifndef R_PPC_ADDR14_BRTAKEN
#define R_PPC_ADDR14_BRTAKEN 8
#endif /* R_PPC_ADDR14_BRTAKEN */
#ifndef R_PPC_ADDR14_BRNTAKEN
#define R_PPC_ADDR14_BRNTAKEN 9
#endif /* R_PPC_ADDR14_BRNTAKEN */
#ifndef R_PPC_REL24
#define R_PPC_REL24          10
#endif /* R_PPC_REL24 */
#ifndef R_PPC_REL14
#define R_PPC_REL14          11
#endif /* R_PPC_REL14 */
#ifndef R_PPC_REL14_BRTAKEN
#define R_PPC_REL14_BRTAKEN  12
#endif /* R_PPC_REL14_BRTAKEN */
#ifndef R_PPC_REL14_BRNTAKEN
#define R_PPC_REL14_BRNTAKEN 13
#endif /* R_PPC_REL14_BRNTAKEN */
#ifndef R_PPC_GOT16
#define R_PPC_GOT16          14
#endif /* R_PPC_GOT16 */
#ifndef R_PPC_GOT16_LO
#define R_PPC_GOT16_LO       15
#endif /* R_PPC_GOT16_LO */
#ifndef R_PPC_GOT16_HI
#define R_PPC_GOT16_HI       16
#endif /* R_PPC_GOT16_HI */
#ifndef R_PPC_GOT16_HA
#define R_PPC_GOT16_HA       17
#endif /* R_PPC_GOT16_HA */
#ifndef R_PPC_PLTREL24
#define R_PPC_PLTREL24       18
#endif /* R_PPC_PLTREL24 */
#ifndef R_PPC_COPY
#define R_PPC_COPY           19
#endif /* R_PPC_COPY */
#ifndef R_PPC_GLOB_DAT
#define R_PPC_GLOB_DAT       20
#endif /* R_PPC_GLOB_DAT */
#ifndef R_PPC_JMP_SLOT
#define R_PPC_JMP_SLOT       21
#endif /* R_PPC_JMP_SLOT */
#ifndef R_PPC_RELATIVE
#define R_PPC_RELATIVE       22
#endif /* R_PPC_RELATIVE */
#ifndef R_PPC_LOCAL24PC
#define R_PPC_LOCAL24PC      23
#endif /* R_PPC_LOCAL24PC */
#ifndef R_PPC_UADDR32
#define R_PPC_UADDR32        24
#endif /* R_PPC_UADDR32 */
#ifndef R_PPC_UADDR16
#define R_PPC_UADDR16        25
#endif /* R_PPC_UADDR16 */
#ifndef R_PPC_REL32
#define R_PPC_REL32          26
#endif /* R_PPC_REL32 */
#ifndef R_PPC_PLT32
#define R_PPC_PLT32          27
#endif /* R_PPC_PLT32 */
#ifndef R_PPC_PLTREL32
#define R_PPC_PLTREL32       28
#endif /* R_PPC_PLTREL32 */
#ifndef R_PPC_PLT16_LO
#define R_PPC_PLT16_LO       29
#endif /* R_PPC_PLT16_LO */
#ifndef R_PPC_PLT16_HI
#define R_PPC_PLT16_HI       30
#endif /* R_PPC_PLT16_HI */
#ifndef R_PPC_PLT16_HA
#define R_PPC_PLT16_HA       31
#endif /* R_PPC_PLT16_HA */
#ifndef R_PPC_SDAREL16
#define R_PPC_SDAREL16       32
#endif /* R_PPC_SDAREL16 */
#ifndef R_PPC_SECTOFF
#define R_PPC_SECTOFF        33
#endif /* R_PPC_SECTOFF */
#ifndef R_PPC_SECTOFF_LO
#define R_PPC_SECTOFF_LO     34
#endif /* R_PPC_SECTOFF_LO */
#ifndef R_PPC_SECTOFF_HI
#define R_PPC_SECTOFF_HI     35
#endif /* R_PPC_SECTOFF_HI */
#ifndef R_PPC_SECTOFF_HA
#define R_PPC_SECTOFF_HA     36
#endif /* R_PPC_SECTOFF_HA */
#ifndef R_PPC_37
#define R_PPC_37             37
#endif /* R_PPC_37 */
#ifndef R_PPC_38
#define R_PPC_38             38
#endif /* R_PPC_38 */
#ifndef R_PPC_39
#define R_PPC_39             39
#endif /* R_PPC_39 */
#ifndef R_PPC_40
#define R_PPC_40             40
#endif /* R_PPC_40 */
#ifndef R_PPC_41
#define R_PPC_41             41
#endif /* R_PPC_41 */
#ifndef R_PPC_42
#define R_PPC_42             42
#endif /* R_PPC_42 */
#ifndef R_PPC_43
#define R_PPC_43             43
#endif /* R_PPC_43 */
#ifndef R_PPC_44
#define R_PPC_44             44
#endif /* R_PPC_44 */
#ifndef R_PPC_45
#define R_PPC_45             45
#endif /* R_PPC_45 */
#ifndef R_PPC_46
#define R_PPC_46             46
#endif /* R_PPC_46 */
#ifndef R_PPC_47
#define R_PPC_47             47
#endif /* R_PPC_47 */
#ifndef R_PPC_48
#define R_PPC_48             48
#endif /* R_PPC_48 */
#ifndef R_PPC_49
#define R_PPC_49             49
#endif /* R_PPC_49 */
#ifndef R_PPC_50
#define R_PPC_50             50
#endif /* R_PPC_50 */
#ifndef R_PPC_51
#define R_PPC_51             51
#endif /* R_PPC_51 */
#ifndef R_PPC_52
#define R_PPC_52             52
#endif /* R_PPC_52 */
#ifndef R_PPC_53
#define R_PPC_53             53
#endif /* R_PPC_53 */
#ifndef R_PPC_54
#define R_PPC_54             54
#endif /* R_PPC_54 */
#ifndef R_PPC_55
#define R_PPC_55             55
#endif /* R_PPC_55 */
#ifndef R_PPC_56
#define R_PPC_56             56
#endif /* R_PPC_56 */
#ifndef R_PPC_57
#define R_PPC_57             57
#endif /* R_PPC_57 */
#ifndef R_PPC_58
#define R_PPC_58             58
#endif /* R_PPC_58 */
#ifndef R_PPC_59
#define R_PPC_59             59
#endif /* R_PPC_59 */
#ifndef R_PPC_60
#define R_PPC_60             60
#endif /* R_PPC_60 */
#ifndef R_PPC_61
#define R_PPC_61             61
#endif /* R_PPC_61 */
#ifndef R_PPC_62
#define R_PPC_62             62
#endif /* R_PPC_62 */
#ifndef R_PPC_63
#define R_PPC_63             63
#endif /* R_PPC_63 */
#ifndef R_PPC_64
#define R_PPC_64             64
#endif /* R_PPC_64 */
#ifndef R_PPC_65
#define R_PPC_65             65
#endif /* R_PPC_65 */
#ifndef R_PPC_66
#define R_PPC_66             66
#endif /* R_PPC_66 */
#ifndef R_PPC_TLS
#define R_PPC_TLS            67
#endif /* R_PPC_TLS */
#ifndef R_PPC_DTPMOD32
#define R_PPC_DTPMOD32       68
#endif /* R_PPC_DTPMOD32 */
#ifndef R_PPC_TPREL16
#define R_PPC_TPREL16        69
#endif /* R_PPC_TPREL16 */
#ifndef R_PPC_TPREL16_LO
#define R_PPC_TPREL16_LO     70
#endif /* R_PPC_TPREL16_LO */
#ifndef R_PPC_TPREL16_HI
#define R_PPC_TPREL16_HI     71
#endif /* R_PPC_TPREL16_HI */
#ifndef R_PPC_TPREL16_HA
#define R_PPC_TPREL16_HA     72
#endif /* R_PPC_TPREL16_HA */
#ifndef R_PPC_TPREL32
#define R_PPC_TPREL32        73
#endif /* R_PPC_TPREL32 */
#ifndef R_PPC_DTPREL16
#define R_PPC_DTPREL16       74
#endif /* R_PPC_DTPREL16 */
#ifndef R_PPC_DTPREL16_LO
#define R_PPC_DTPREL16_LO    75
#endif /* R_PPC_DTPREL16_LO */
#ifndef R_PPC_DTPREL16_HI
#define R_PPC_DTPREL16_HI    76
#endif /* R_PPC_DTPREL16_HI */
#ifndef R_PPC_DTPREL16_HA
#define R_PPC_DTPREL16_HA    77
#endif /* R_PPC_DTPREL16_HA */
#ifndef R_PPC_DTPREL32
#define R_PPC_DTPREL32       78
#endif /* R_PPC_DTPREL32 */
#ifndef R_PPC_GOT_TLSGD16
#define R_PPC_GOT_TLSGD16    79
#endif /* R_PPC_GOT_TLSGD16 */
#ifndef R_PPC_GOT_TLSGD16_LO
#define R_PPC_GOT_TLSGD16_LO 80
#endif /* R_PPC_GOT_TLSGD16_LO */
#ifndef R_PPC_GOT_TLSGD16_HI
#define R_PPC_GOT_TLSGD16_HI 81
#endif /* R_PPC_GOT_TLSGD16_HI */
#ifndef R_PPC_GOT_TLSGD16_HA
#define R_PPC_GOT_TLSGD16_HA 82
#endif /* R_PPC_GOT_TLSGD16_HA */
#ifndef R_PPC_GOT_TLSLD16
#define R_PPC_GOT_TLSLD16    83
#endif /* R_PPC_GOT_TLSLD16 */
#ifndef R_PPC_GOT_TLSLD16_LO
#define R_PPC_GOT_TLSLD16_LO 84
#endif /* R_PPC_GOT_TLSLD16_LO */
#ifndef R_PPC_GOT_TLSLD16_HI
#define R_PPC_GOT_TLSLD16_HI 85
#endif /* R_PPC_GOT_TLSLD16_HI */
#ifndef R_PPC_GOT_TLSLD16_HA
#define R_PPC_GOT_TLSLD16_HA 86
#endif /* R_PPC_GOT_TLSLD16_HA */
#ifndef R_PPC_GOT_TPREL16
#define R_PPC_GOT_TPREL16    87
#endif /* R_PPC_GOT_TPREL16 */
#ifndef R_PPC_GOT_TPREL16_LO
#define R_PPC_GOT_TPREL16_LO 88
#endif /* R_PPC_GOT_TPREL16_LO */
#ifndef R_PPC_GOT_TPREL16_HI
#define R_PPC_GOT_TPREL16_HI 89
#endif /* R_PPC_GOT_TPREL16_HI */
#ifndef R_PPC_GOT_TPREL16_HA
#define R_PPC_GOT_TPREL16_HA 90
#endif /* R_PPC_GOT_TPREL16_HA */
#ifndef R_PPC_GOT_DTPREL16
#define R_PPC_GOT_DTPREL16   91
#endif /* R_PPC_GOT_DTPREL16 */
#ifndef R_PPC_GOT_DTPREL16_LO
#define R_PPC_GOT_DTPREL16_LO 92
#endif /* R_PPC_GOT_DTPREL16_LO */
#ifndef R_PPC_GOT_DTPREL16_HI
#define R_PPC_GOT_DTPREL16_HI 93
#endif /* R_PPC_GOT_DTPREL16_HI */
#ifndef R_PPC_GOT_DTPREL16_HA
#define R_PPC_GOT_DTPREL16_HA 94
#endif /* R_PPC_GOT_DTPREL16_HA */
#ifndef R_PPC_TLSGD
#define R_PPC_TLSGD          95
#endif /* R_PPC_TLSGD */
#ifndef R_PPC_TLSLD
#define R_PPC_TLSLD          96
#endif /* R_PPC_TLSLD */
#ifndef R_PPC_EMB_NADDR32
#define R_PPC_EMB_NADDR32    101
#endif /* R_PPC_EMB_NADDR32 */
#ifndef R_PPC_EMB_NADDR16
#define R_PPC_EMB_NADDR16    102
#endif /* R_PPC_EMB_NADDR16 */
#ifndef R_PPC_EMB_NADDR16_LO
#define R_PPC_EMB_NADDR16_LO 103
#endif /* R_PPC_EMB_NADDR16_LO */
#ifndef R_PPC_EMB_NADDR16_HI
#define R_PPC_EMB_NADDR16_HI 104
#endif /* R_PPC_EMB_NADDR16_HI */
#ifndef R_PPC_EMB_NADDR16_HA
#define R_PPC_EMB_NADDR16_HA 105
#endif /* R_PPC_EMB_NADDR16_HA */
#ifndef R_PPC_EMB_SDAI16
#define R_PPC_EMB_SDAI16     106
#endif /* R_PPC_EMB_SDAI16 */
#ifndef R_PPC_EMB_SDA2I16
#define R_PPC_EMB_SDA2I16    107
#endif /* R_PPC_EMB_SDA2I16 */
#ifndef R_PPC_EMB_SDA2REL
#define R_PPC_EMB_SDA2REL    108
#endif /* R_PPC_EMB_SDA2REL */
#ifndef R_PPC_EMB_SDA21
#define R_PPC_EMB_SDA21      109
#endif /* R_PPC_EMB_SDA21 */
#ifndef R_PPC_EMB_MRKREF
#define R_PPC_EMB_MRKREF     110
#endif /* R_PPC_EMB_MRKREF */
#ifndef R_PPC_EMB_RELSEC16
#define R_PPC_EMB_RELSEC16   111
#endif /* R_PPC_EMB_RELSEC16 */
#ifndef R_PPC_EMB_RELST_LO
#define R_PPC_EMB_RELST_LO   112
#endif /* R_PPC_EMB_RELST_LO */
#ifndef R_PPC_EMB_RELST_HI
#define R_PPC_EMB_RELST_HI   113
#endif /* R_PPC_EMB_RELST_HI */
#ifndef R_PPC_EMB_RELST_HA
#define R_PPC_EMB_RELST_HA   114
#endif /* R_PPC_EMB_RELST_HA */
#ifndef R_PPC_EMB_BIT_FLD
#define R_PPC_EMB_BIT_FLD    115
#endif /* R_PPC_EMB_BIT_FLD */
#ifndef R_PPC_EMB_RELSDA
#define R_PPC_EMB_RELSDA     116
#endif /* R_PPC_EMB_RELSDA */
#ifndef R_PPC_DIAB_SDA21_LO
#define R_PPC_DIAB_SDA21_LO  180
#endif /* R_PPC_DIAB_SDA21_LO */
#ifndef R_PPC_DIAB_SDA21_HI
#define R_PPC_DIAB_SDA21_HI  181
#endif /* R_PPC_DIAB_SDA21_HI */
#ifndef R_PPC_DIAB_SDA21_HA
#define R_PPC_DIAB_SDA21_HA  182
#endif /* R_PPC_DIAB_SDA21_HA */
#ifndef R_PPC_DIAB_RELSDA_LO
#define R_PPC_DIAB_RELSDA_LO 183
#endif /* R_PPC_DIAB_RELSDA_LO */
#ifndef R_PPC_DIAB_RELSDA_HI
#define R_PPC_DIAB_RELSDA_HI 184
#endif /* R_PPC_DIAB_RELSDA_HI */
#ifndef R_PPC_DIAB_RELSDA_HA
#define R_PPC_DIAB_RELSDA_HA 185
#endif /* R_PPC_DIAB_RELSDA_HA */
#ifndef R_PPC_IRELATIVE
#define R_PPC_IRELATIVE      248
#endif /* R_PPC_IRELATIVE */
#ifndef R_PPC_REL16
#define R_PPC_REL16          249
#endif /* R_PPC_REL16 */
#ifndef R_PPC_REL16_LO
#define R_PPC_REL16_LO       250
#endif /* R_PPC_REL16_LO */
#ifndef R_PPC_REL16_HI
#define R_PPC_REL16_HI       251
#endif /* R_PPC_REL16_HI */
#ifndef R_PPC_REL16_HA
#define R_PPC_REL16_HA       252
#endif /* R_PPC_REL16_HA */
