/* Automatically generated, do not edit. */
/* Generated sourcedate  2020-11-23 18:16:30-08:00   */

/* BEGIN FILE */

#ifndef __DWARF_NAMES_ENUM_H__
#define __DWARF_NAMES_ENUM_H__

enum Dwarf_TAG_e {
    DW_TAG_array_type                        = 0x0001,
    DW_TAG_class_type                        = 0x0002,
    DW_TAG_entry_point                       = 0x0003,
    DW_TAG_enumeration_type                  = 0x0004,
    DW_TAG_formal_parameter                  = 0x0005,
    DW_TAG_imported_declaration              = 0x0008,
    DW_TAG_label                             = 0x000a,
    DW_TAG_lexical_block                     = 0x000b,
    DW_TAG_member                            = 0x000d,
    DW_TAG_pointer_type                      = 0x000f,
    DW_TAG_reference_type                    = 0x0010,
    DW_TAG_compile_unit                      = 0x0011,
    DW_TAG_string_type                       = 0x0012,
    DW_TAG_structure_type                    = 0x0013,
    DW_TAG_subroutine_type                   = 0x0015,
    DW_TAG_typedef                           = 0x0016,
    DW_TAG_union_type                        = 0x0017,
    DW_TAG_unspecified_parameters            = 0x0018,
    DW_TAG_variant                           = 0x0019,
    DW_TAG_common_block                      = 0x001a,
    DW_TAG_common_inclusion                  = 0x001b,
    DW_TAG_inheritance                       = 0x001c,
    DW_TAG_inlined_subroutine                = 0x001d,
    DW_TAG_module                            = 0x001e,
    DW_TAG_ptr_to_member_type                = 0x001f,
    DW_TAG_set_type                          = 0x0020,
    DW_TAG_subrange_type                     = 0x0021,
    DW_TAG_with_stmt                         = 0x0022,
    DW_TAG_access_declaration                = 0x0023,
    DW_TAG_base_type                         = 0x0024,
    DW_TAG_catch_block                       = 0x0025,
    DW_TAG_const_type                        = 0x0026,
    DW_TAG_constant                          = 0x0027,
    DW_TAG_enumerator                        = 0x0028,
    DW_TAG_file_type                         = 0x0029,
    DW_TAG_friend                            = 0x002a,
    DW_TAG_namelist                          = 0x002b,
    DW_TAG_namelist_item                     = 0x002c,
    DW_TAG_packed_type                       = 0x002d,
    DW_TAG_subprogram                        = 0x002e,
    DW_TAG_template_type_parameter           = 0x002f,
    DW_TAG_template_value_parameter          = 0x0030,
    DW_TAG_thrown_type                       = 0x0031,
    DW_TAG_try_block                         = 0x0032,
    DW_TAG_variant_part                      = 0x0033,
    DW_TAG_variable                          = 0x0034,
    DW_TAG_volatile_type                     = 0x0035,
    DW_TAG_dwarf_procedure                   = 0x0036,
    DW_TAG_restrict_type                     = 0x0037,
    DW_TAG_interface_type                    = 0x0038,
    DW_TAG_namespace                         = 0x0039,
    DW_TAG_imported_module                   = 0x003a,
    DW_TAG_unspecified_type                  = 0x003b,
    DW_TAG_partial_unit                      = 0x003c,
    DW_TAG_imported_unit                     = 0x003d,
    DW_TAG_mutable_type                      = 0x003e,
    DW_TAG_condition                         = 0x003f,
    DW_TAG_shared_type                       = 0x0040,
    DW_TAG_type_unit                         = 0x0041,
    DW_TAG_rvalue_reference_type             = 0x0042,
    DW_TAG_template_alias                    = 0x0043,
    DW_TAG_coarray_type                      = 0x0044,
    DW_TAG_generic_subrange                  = 0x0045,
    DW_TAG_dynamic_type                      = 0x0046,
    DW_TAG_atomic_type                       = 0x0047,
    DW_TAG_call_site                         = 0x0048,
    DW_TAG_call_site_parameter               = 0x0049,
    DW_TAG_skeleton_unit                     = 0x004a,
    DW_TAG_immutable_type                    = 0x004b,
    DW_TAG_lo_user                           = 0x4080,
    DW_TAG_MIPS_loop                         = 0x4081,
    DW_TAG_HP_array_descriptor               = 0x4090,
    DW_TAG_format_label                      = 0x4101,
    DW_TAG_function_template                 = 0x4102,
    DW_TAG_class_template                    = 0x4103,
    DW_TAG_GNU_BINCL                         = 0x4104,
    DW_TAG_GNU_EINCL                         = 0x4105,
    DW_TAG_GNU_template_template_parameter   = 0x4106,
    DW_TAG_GNU_template_parameter_pack       = 0x4107,
    DW_TAG_GNU_formal_parameter_pack         = 0x4108,
    DW_TAG_GNU_call_site                     = 0x4109,
    DW_TAG_GNU_call_site_parameter           = 0x410a,
    DW_TAG_SUN_function_template             = 0x4201,
    DW_TAG_SUN_class_template                = 0x4202,
    DW_TAG_SUN_struct_template               = 0x4203,
    DW_TAG_SUN_union_template                = 0x4204,
    DW_TAG_SUN_indirect_inheritance          = 0x4205,
    DW_TAG_SUN_codeflags                     = 0x4206,
    DW_TAG_SUN_memop_info                    = 0x4207,
    DW_TAG_SUN_omp_child_func                = 0x4208,
    DW_TAG_SUN_rtti_descriptor               = 0x4209,
    DW_TAG_SUN_dtor_info                     = 0x420a,
    DW_TAG_SUN_dtor                          = 0x420b,
    DW_TAG_SUN_f90_interface                 = 0x420c,
    DW_TAG_SUN_fortran_vax_structure         = 0x420d,
    DW_TAG_SUN_hi                            = 0x42ff,
    DW_TAG_ALTIUM_circ_type                  = 0x5101,
    DW_TAG_ALTIUM_mwa_circ_type              = 0x5102,
    DW_TAG_ALTIUM_rev_carry_type             = 0x5103,
    DW_TAG_ALTIUM_rom                        = 0x5111,
    DW_TAG_upc_shared_type                   = 0x8765,
    DW_TAG_upc_strict_type                   = 0x8766,
    DW_TAG_upc_relaxed_type                  = 0x8767,
    DW_TAG_PGI_kanji_type                    = 0xa000,
    DW_TAG_PGI_interface_block               = 0xa020,
    DW_TAG_hi_user                           = 0xffff
};

enum Dwarf_children_e {
    DW_children_no                           = 0x0000,
    DW_children_yes                          = 0x0001
};

enum Dwarf_FORM_e {
    DW_FORM_addr                             = 0x0001,
    DW_FORM_block2                           = 0x0003,
    DW_FORM_block4                           = 0x0004,
    DW_FORM_data2                            = 0x0005,
    DW_FORM_data4                            = 0x0006,
    DW_FORM_data8                            = 0x0007,
    DW_FORM_string                           = 0x0008,
    DW_FORM_block                            = 0x0009,
    DW_FORM_block1                           = 0x000a,
    DW_FORM_data1                            = 0x000b,
    DW_FORM_flag                             = 0x000c,
    DW_FORM_sdata                            = 0x000d,
    DW_FORM_strp                             = 0x000e,
    DW_FORM_udata                            = 0x000f,
    DW_FORM_ref_addr                         = 0x0010,
    DW_FORM_ref1                             = 0x0011,
    DW_FORM_ref2                             = 0x0012,
    DW_FORM_ref4                             = 0x0013,
    DW_FORM_ref8                             = 0x0014,
    DW_FORM_ref_udata                        = 0x0015,
    DW_FORM_indirect                         = 0x0016,
    DW_FORM_sec_offset                       = 0x0017,
    DW_FORM_exprloc                          = 0x0018,
    DW_FORM_flag_present                     = 0x0019,
    DW_FORM_strx                             = 0x001a,
    DW_FORM_addrx                            = 0x001b,
    DW_FORM_ref_sup4                         = 0x001c,
    DW_FORM_strp_sup                         = 0x001d,
    DW_FORM_data16                           = 0x001e,
    DW_FORM_line_strp                        = 0x001f,
    DW_FORM_ref_sig8                         = 0x0020,
    DW_FORM_implicit_const                   = 0x0021,
    DW_FORM_loclistx                         = 0x0022,
    DW_FORM_rnglistx                         = 0x0023,
    DW_FORM_ref_sup8                         = 0x0024,
    DW_FORM_strx1                            = 0x0025,
    DW_FORM_strx2                            = 0x0026,
    DW_FORM_strx3                            = 0x0027,
    DW_FORM_strx4                            = 0x0028,
    DW_FORM_addrx1                           = 0x0029,
    DW_FORM_addrx2                           = 0x002a,
    DW_FORM_addrx3                           = 0x002b,
    DW_FORM_addrx4                           = 0x002c,
    DW_FORM_GNU_addr_index                   = 0x1f01,
    DW_FORM_GNU_str_index                    = 0x1f02,
    DW_FORM_GNU_ref_alt                      = 0x1f20,
    DW_FORM_GNU_strp_alt                     = 0x1f21
};

enum Dwarf_AT_e {
    DW_AT_sibling                            = 0x0001,
    DW_AT_location                           = 0x0002,
    DW_AT_name                               = 0x0003,
    DW_AT_ordering                           = 0x0009,
    DW_AT_subscr_data                        = 0x000a,
    DW_AT_byte_size                          = 0x000b,
    DW_AT_bit_offset                         = 0x000c,
    DW_AT_bit_size                           = 0x000d,
    DW_AT_element_list                       = 0x000f,
    DW_AT_stmt_list                          = 0x0010,
    DW_AT_low_pc                             = 0x0011,
    DW_AT_high_pc                            = 0x0012,
    DW_AT_language                           = 0x0013,
    DW_AT_member                             = 0x0014,
    DW_AT_discr                              = 0x0015,
    DW_AT_discr_value                        = 0x0016,
    DW_AT_visibility                         = 0x0017,
    DW_AT_import                             = 0x0018,
    DW_AT_string_length                      = 0x0019,
    DW_AT_common_reference                   = 0x001a,
    DW_AT_comp_dir                           = 0x001b,
    DW_AT_const_value                        = 0x001c,
    DW_AT_containing_type                    = 0x001d,
    DW_AT_default_value                      = 0x001e,
    DW_AT_inline                             = 0x0020,
    DW_AT_is_optional                        = 0x0021,
    DW_AT_lower_bound                        = 0x0022,
    DW_AT_producer                           = 0x0025,
    DW_AT_prototyped                         = 0x0027,
    DW_AT_return_addr                        = 0x002a,
    DW_AT_start_scope                        = 0x002c,
    DW_AT_bit_stride                         = 0x002e,
    DW_AT_upper_bound                        = 0x002f,
    DW_AT_abstract_origin                    = 0x0031,
    DW_AT_accessibility                      = 0x0032,
    DW_AT_address_class                      = 0x0033,
    DW_AT_artificial                         = 0x0034,
    DW_AT_base_types                         = 0x0035,
    DW_AT_calling_convention                 = 0x0036,
    DW_AT_count                              = 0x0037,
    DW_AT_data_member_location               = 0x0038,
    DW_AT_decl_column                        = 0x0039,
    DW_AT_decl_file                          = 0x003a,
    DW_AT_decl_line                          = 0x003b,
    DW_AT_declaration                        = 0x003c,
    DW_AT_discr_list                         = 0x003d,
    DW_AT_encoding                           = 0x003e,
    DW_AT_external                           = 0x003f,
    DW_AT_frame_base                         = 0x0040,
    DW_AT_friend                             = 0x0041,
    DW_AT_identifier_case                    = 0x0042,
    DW_AT_macro_info                         = 0x0043,
    DW_AT_namelist_item                      = 0x0044,
    DW_AT_priority                           = 0x0045,
    DW_AT_segment                            = 0x0046,
    DW_AT_specification                      = 0x0047,
    DW_AT_static_link                        = 0x0048,
    DW_AT_type                               = 0x0049,
    DW_AT_use_location                       = 0x004a,
    DW_AT_variable_parameter                 = 0x004b,
    DW_AT_virtuality                         = 0x004c,
    DW_AT_vtable_elem_location               = 0x004d,
    DW_AT_allocated                          = 0x004e,
    DW_AT_associated                         = 0x004f,
    DW_AT_data_location                      = 0x0050,
    DW_AT_byte_stride                        = 0x0051,
    DW_AT_entry_pc                           = 0x0052,
    DW_AT_use_UTF8                           = 0x0053,
    DW_AT_extension                          = 0x0054,
    DW_AT_ranges                             = 0x0055,
    DW_AT_trampoline                         = 0x0056,
    DW_AT_call_column                        = 0x0057,
    DW_AT_call_file                          = 0x0058,
    DW_AT_call_line                          = 0x0059,
    DW_AT_description                        = 0x005a,
    DW_AT_binary_scale                       = 0x005b,
    DW_AT_decimal_scale                      = 0x005c,
    DW_AT_small                              = 0x005d,
    DW_AT_decimal_sign                       = 0x005e,
    DW_AT_digit_count                        = 0x005f,
    DW_AT_picture_string                     = 0x0060,
    DW_AT_mutable                            = 0x0061,
    DW_AT_threads_scaled                     = 0x0062,
    DW_AT_explicit                           = 0x0063,
    DW_AT_object_pointer                     = 0x0064,
    DW_AT_endianity                          = 0x0065,
    DW_AT_elemental                          = 0x0066,
    DW_AT_pure                               = 0x0067,
    DW_AT_recursive                          = 0x0068,
    DW_AT_signature                          = 0x0069,
    DW_AT_main_subprogram                    = 0x006a,
    DW_AT_data_bit_offset                    = 0x006b,
    DW_AT_const_expr                         = 0x006c,
    DW_AT_enum_class                         = 0x006d,
    DW_AT_linkage_name                       = 0x006e,
    DW_AT_string_length_bit_size             = 0x006f,
    DW_AT_string_length_byte_size            = 0x0070,
    DW_AT_rank                               = 0x0071,
    DW_AT_str_offsets_base                   = 0x0072,
    DW_AT_addr_base                          = 0x0073,
    DW_AT_rnglists_base                      = 0x0074,
    DW_AT_dwo_id                             = 0x0075,
    DW_AT_dwo_name                           = 0x0076,
    DW_AT_reference                          = 0x0077,
    DW_AT_rvalue_reference                   = 0x0078,
    DW_AT_macros                             = 0x0079,
    DW_AT_call_all_calls                     = 0x007a,
    DW_AT_call_all_source_calls              = 0x007b,
    DW_AT_call_all_tail_calls                = 0x007c,
    DW_AT_call_return_pc                     = 0x007d,
    DW_AT_call_value                         = 0x007e,
    DW_AT_call_origin                        = 0x007f,
    DW_AT_call_parameter                     = 0x0080,
    DW_AT_call_pc                            = 0x0081,
    DW_AT_call_tail_call                     = 0x0082,
    DW_AT_call_target                        = 0x0083,
    DW_AT_call_target_clobbered              = 0x0084,
    DW_AT_call_data_location                 = 0x0085,
    DW_AT_call_data_value                    = 0x0086,
    DW_AT_noreturn                           = 0x0087,
    DW_AT_alignment                          = 0x0088,
    DW_AT_export_symbols                     = 0x0089,
    DW_AT_deleted                            = 0x008a,
    DW_AT_defaulted                          = 0x008b,
    DW_AT_loclists_base                      = 0x008c,
    DW_AT_HP_block_index                     = 0x2000,
    DW_AT_MIPS_fde                           = 0x2001,
    DW_AT_MIPS_loop_begin                    = 0x2002,
    DW_AT_MIPS_tail_loop_begin               = 0x2003,
    DW_AT_MIPS_epilog_begin                  = 0x2004,
    DW_AT_MIPS_loop_unroll_factor            = 0x2005,
    DW_AT_MIPS_software_pipeline_depth       = 0x2006,
    DW_AT_MIPS_linkage_name                  = 0x2007,
    DW_AT_MIPS_stride                        = 0x2008,
    DW_AT_MIPS_abstract_name                 = 0x2009,
    DW_AT_MIPS_clone_origin                  = 0x200a,
    DW_AT_MIPS_has_inlines                   = 0x200b,
    DW_AT_MIPS_stride_byte                   = 0x200c,
    DW_AT_MIPS_stride_elem                   = 0x200d,
    DW_AT_MIPS_ptr_dopetype                  = 0x200e,
    DW_AT_MIPS_allocatable_dopetype          = 0x200f,
    DW_AT_MIPS_assumed_shape_dopetype        = 0x2010,
    DW_AT_MIPS_assumed_size                  = 0x2011,
    DW_AT_HP_raw_data_ptr                    = 0x2012,
    DW_AT_HP_pass_by_reference               = 0x2013,
    DW_AT_HP_opt_level                       = 0x2014,
    DW_AT_HP_prof_version_id                 = 0x2015,
    DW_AT_HP_opt_flags                       = 0x2016,
    DW_AT_HP_cold_region_low_pc              = 0x2017,
    DW_AT_HP_cold_region_high_pc             = 0x2018,
    DW_AT_HP_all_variables_modifiable        = 0x2019,
    DW_AT_HP_linkage_name                    = 0x201a,
    DW_AT_HP_prof_flags                      = 0x201b,
    DW_AT_INTEL_other_endian                 = 0x2026,
    DW_AT_sf_names                           = 0x2101,
    DW_AT_src_info                           = 0x2102,
    DW_AT_mac_info                           = 0x2103,
    DW_AT_src_coords                         = 0x2104,
    DW_AT_body_begin                         = 0x2105,
    DW_AT_body_end                           = 0x2106,
    DW_AT_GNU_vector                         = 0x2107,
    DW_AT_GNU_guarded_by                     = 0x2108,
    DW_AT_GNU_pt_guarded_by                  = 0x2109,
    DW_AT_GNU_guarded                        = 0x210a,
    DW_AT_GNU_pt_guarded                     = 0x210b,
    DW_AT_GNU_locks_excluded                 = 0x210c,
    DW_AT_GNU_exclusive_locks_required       = 0x210d,
    DW_AT_GNU_shared_locks_required          = 0x210e,
    DW_AT_GNU_odr_signature                  = 0x210f,
    DW_AT_GNU_template_name                  = 0x2110,
    DW_AT_GNU_call_site_value                = 0x2111,
    DW_AT_GNU_call_site_data_value           = 0x2112,
    DW_AT_GNU_call_site_target               = 0x2113,
    DW_AT_GNU_call_site_target_clobbered     = 0x2114,
    DW_AT_GNU_tail_call                      = 0x2115,
    DW_AT_GNU_all_tail_call_sites            = 0x2116,
    DW_AT_GNU_all_call_sites                 = 0x2117,
    DW_AT_GNU_all_source_call_sites          = 0x2118,
    DW_AT_GNU_macros                         = 0x2119,
    DW_AT_GNU_deleted                        = 0x211a,
    DW_AT_GNU_dwo_name                       = 0x2130,
    DW_AT_GNU_dwo_id                         = 0x2131,
    DW_AT_GNU_ranges_base                    = 0x2132,
    DW_AT_GNU_addr_base                      = 0x2133,
    DW_AT_GNU_pubnames                       = 0x2134,
    DW_AT_GNU_pubtypes                       = 0x2135,
    DW_AT_GNU_discriminator                  = 0x2136,
    DW_AT_GNU_locviews                       = 0x2137,
    DW_AT_GNU_entry_view                     = 0x2138,
    DW_AT_SUN_template                       = 0x2201,
    DW_AT_SUN_alignment                      = 0x2202,
    DW_AT_SUN_vtable                         = 0x2203,
    DW_AT_SUN_count_guarantee                = 0x2204,
    DW_AT_SUN_command_line                   = 0x2205,
    DW_AT_SUN_vbase                          = 0x2206,
    DW_AT_SUN_compile_options                = 0x2207,
    DW_AT_SUN_language                       = 0x2208,
    DW_AT_SUN_browser_file                   = 0x2209,
    DW_AT_SUN_vtable_abi                     = 0x2210,
    DW_AT_SUN_func_offsets                   = 0x2211,
    DW_AT_SUN_cf_kind                        = 0x2212,
    DW_AT_SUN_vtable_index                   = 0x2213,
    DW_AT_SUN_omp_tpriv_addr                 = 0x2214,
    DW_AT_SUN_omp_child_func                 = 0x2215,
    DW_AT_SUN_func_offset                    = 0x2216,
    DW_AT_SUN_memop_type_ref                 = 0x2217,
    DW_AT_SUN_profile_id                     = 0x2218,
    DW_AT_SUN_memop_signature                = 0x2219,
    DW_AT_SUN_obj_dir                        = 0x2220,
    DW_AT_SUN_obj_file                       = 0x2221,
    DW_AT_SUN_original_name                  = 0x2222,
    DW_AT_SUN_hwcprof_signature              = 0x2223,
    DW_AT_SUN_amd64_parmdump                 = 0x2224,
    DW_AT_SUN_part_link_name                 = 0x2225,
    DW_AT_SUN_link_name                      = 0x2226,
    DW_AT_SUN_pass_with_const                = 0x2227,
    DW_AT_SUN_return_with_const              = 0x2228,
    DW_AT_SUN_import_by_name                 = 0x2229,
    DW_AT_SUN_f90_pointer                    = 0x222a,
    DW_AT_SUN_pass_by_ref                    = 0x222b,
    DW_AT_SUN_f90_allocatable                = 0x222c,
    DW_AT_SUN_f90_assumed_shape_array        = 0x222d,
    DW_AT_SUN_c_vla                          = 0x222e,
    DW_AT_SUN_return_value_ptr               = 0x2230,
    DW_AT_SUN_dtor_start                     = 0x2231,
    DW_AT_SUN_dtor_length                    = 0x2232,
    DW_AT_SUN_dtor_state_initial             = 0x2233,
    DW_AT_SUN_dtor_state_final               = 0x2234,
    DW_AT_SUN_dtor_state_deltas              = 0x2235,
    DW_AT_SUN_import_by_lname                = 0x2236,
    DW_AT_SUN_f90_use_only                   = 0x2237,
    DW_AT_SUN_namelist_spec                  = 0x2238,
    DW_AT_SUN_is_omp_child_func              = 0x2239,
    DW_AT_SUN_fortran_main_alias             = 0x223a,
    DW_AT_SUN_fortran_based                  = 0x223b,
    DW_AT_ALTIUM_loclist                     = 0x2300,
    DW_AT_use_GNAT_descriptive_type          = 0x2301,
    DW_AT_GNAT_descriptive_type              = 0x2302,
    DW_AT_GNU_numerator                      = 0x2303,
    DW_AT_GNU_denominator                    = 0x2304,
    DW_AT_GNU_bias                           = 0x2305,
    DW_AT_go_kind                            = 0x2900,
    DW_AT_go_key                             = 0x2901,
    DW_AT_go_elem                            = 0x2902,
    DW_AT_go_embedded_field                  = 0x2903,
    DW_AT_go_runtime_type                    = 0x2904,
    DW_AT_upc_threads_scaled                 = 0x3210,
    DW_AT_PGI_lbase                          = 0x3a00,
    DW_AT_PGI_soffset                        = 0x3a01,
    DW_AT_PGI_lstride                        = 0x3a02,
    DW_AT_APPLE_optimized                    = 0x3fe1,
    DW_AT_APPLE_flags                        = 0x3fe2,
    DW_AT_APPLE_isa                          = 0x3fe3,
    DW_AT_APPLE_block                        = 0x3fe4,
    DW_AT_APPLE_major_runtime_vers           = 0x3fe5,
    DW_AT_APPLE_runtime_class                = 0x3fe6,
    DW_AT_APPLE_omit_frame_ptr               = 0x3fe7,
    DW_AT_hi_user                            = 0x3fff
};

enum Dwarf_OP_e {
    DW_OP_addr                               = 0x0003,
    DW_OP_deref                              = 0x0006,
    DW_OP_const1u                            = 0x0008,
    DW_OP_const1s                            = 0x0009,
    DW_OP_const2u                            = 0x000a,
    DW_OP_const2s                            = 0x000b,
    DW_OP_const4u                            = 0x000c,
    DW_OP_const4s                            = 0x000d,
    DW_OP_const8u                            = 0x000e,
    DW_OP_const8s                            = 0x000f,
    DW_OP_constu                             = 0x0010,
    DW_OP_consts                             = 0x0011,
    DW_OP_dup                                = 0x0012,
    DW_OP_drop                               = 0x0013,
    DW_OP_over                               = 0x0014,
    DW_OP_pick                               = 0x0015,
    DW_OP_swap                               = 0x0016,
    DW_OP_rot                                = 0x0017,
    DW_OP_xderef                             = 0x0018,
    DW_OP_abs                                = 0x0019,
    DW_OP_and                                = 0x001a,
    DW_OP_div                                = 0x001b,
    DW_OP_minus                              = 0x001c,
    DW_OP_mod                                = 0x001d,
    DW_OP_mul                                = 0x001e,
    DW_OP_neg                                = 0x001f,
    DW_OP_not                                = 0x0020,
    DW_OP_or                                 = 0x0021,
    DW_OP_plus                               = 0x0022,
    DW_OP_plus_uconst                        = 0x0023,
    DW_OP_shl                                = 0x0024,
    DW_OP_shr                                = 0x0025,
    DW_OP_shra                               = 0x0026,
    DW_OP_xor                                = 0x0027,
    DW_OP_bra                                = 0x0028,
    DW_OP_eq                                 = 0x0029,
    DW_OP_ge                                 = 0x002a,
    DW_OP_gt                                 = 0x002b,
    DW_OP_le                                 = 0x002c,
    DW_OP_lt                                 = 0x002d,
    DW_OP_ne                                 = 0x002e,
    DW_OP_skip                               = 0x002f,
    DW_OP_lit0                               = 0x0030,
    DW_OP_lit1                               = 0x0031,
    DW_OP_lit2                               = 0x0032,
    DW_OP_lit3                               = 0x0033,
    DW_OP_lit4                               = 0x0034,
    DW_OP_lit5                               = 0x0035,
    DW_OP_lit6                               = 0x0036,
    DW_OP_lit7                               = 0x0037,
    DW_OP_lit8                               = 0x0038,
    DW_OP_lit9                               = 0x0039,
    DW_OP_lit10                              = 0x003a,
    DW_OP_lit11                              = 0x003b,
    DW_OP_lit12                              = 0x003c,
    DW_OP_lit13                              = 0x003d,
    DW_OP_lit14                              = 0x003e,
    DW_OP_lit15                              = 0x003f,
    DW_OP_lit16                              = 0x0040,
    DW_OP_lit17                              = 0x0041,
    DW_OP_lit18                              = 0x0042,
    DW_OP_lit19                              = 0x0043,
    DW_OP_lit20                              = 0x0044,
    DW_OP_lit21                              = 0x0045,
    DW_OP_lit22                              = 0x0046,
    DW_OP_lit23                              = 0x0047,
    DW_OP_lit24                              = 0x0048,
    DW_OP_lit25                              = 0x0049,
    DW_OP_lit26                              = 0x004a,
    DW_OP_lit27                              = 0x004b,
    DW_OP_lit28                              = 0x004c,
    DW_OP_lit29                              = 0x004d,
    DW_OP_lit30                              = 0x004e,
    DW_OP_lit31                              = 0x004f,
    DW_OP_reg0                               = 0x0050,
    DW_OP_reg1                               = 0x0051,
    DW_OP_reg2                               = 0x0052,
    DW_OP_reg3                               = 0x0053,
    DW_OP_reg4                               = 0x0054,
    DW_OP_reg5                               = 0x0055,
    DW_OP_reg6                               = 0x0056,
    DW_OP_reg7                               = 0x0057,
    DW_OP_reg8                               = 0x0058,
    DW_OP_reg9                               = 0x0059,
    DW_OP_reg10                              = 0x005a,
    DW_OP_reg11                              = 0x005b,
    DW_OP_reg12                              = 0x005c,
    DW_OP_reg13                              = 0x005d,
    DW_OP_reg14                              = 0x005e,
    DW_OP_reg15                              = 0x005f,
    DW_OP_reg16                              = 0x0060,
    DW_OP_reg17                              = 0x0061,
    DW_OP_reg18                              = 0x0062,
    DW_OP_reg19                              = 0x0063,
    DW_OP_reg20                              = 0x0064,
    DW_OP_reg21                              = 0x0065,
    DW_OP_reg22                              = 0x0066,
    DW_OP_reg23                              = 0x0067,
    DW_OP_reg24                              = 0x0068,
    DW_OP_reg25                              = 0x0069,
    DW_OP_reg26                              = 0x006a,
    DW_OP_reg27                              = 0x006b,
    DW_OP_reg28                              = 0x006c,
    DW_OP_reg29                              = 0x006d,
    DW_OP_reg30                              = 0x006e,
    DW_OP_reg31                              = 0x006f,
    DW_OP_breg0                              = 0x0070,
    DW_OP_breg1                              = 0x0071,
    DW_OP_breg2                              = 0x0072,
    DW_OP_breg3                              = 0x0073,
    DW_OP_breg4                              = 0x0074,
    DW_OP_breg5                              = 0x0075,
    DW_OP_breg6                              = 0x0076,
    DW_OP_breg7                              = 0x0077,
    DW_OP_breg8                              = 0x0078,
    DW_OP_breg9                              = 0x0079,
    DW_OP_breg10                             = 0x007a,
    DW_OP_breg11                             = 0x007b,
    DW_OP_breg12                             = 0x007c,
    DW_OP_breg13                             = 0x007d,
    DW_OP_breg14                             = 0x007e,
    DW_OP_breg15                             = 0x007f,
    DW_OP_breg16                             = 0x0080,
    DW_OP_breg17                             = 0x0081,
    DW_OP_breg18                             = 0x0082,
    DW_OP_breg19                             = 0x0083,
    DW_OP_breg20                             = 0x0084,
    DW_OP_breg21                             = 0x0085,
    DW_OP_breg22                             = 0x0086,
    DW_OP_breg23                             = 0x0087,
    DW_OP_breg24                             = 0x0088,
    DW_OP_breg25                             = 0x0089,
    DW_OP_breg26                             = 0x008a,
    DW_OP_breg27                             = 0x008b,
    DW_OP_breg28                             = 0x008c,
    DW_OP_breg29                             = 0x008d,
    DW_OP_breg30                             = 0x008e,
    DW_OP_breg31                             = 0x008f,
    DW_OP_regx                               = 0x0090,
    DW_OP_fbreg                              = 0x0091,
    DW_OP_bregx                              = 0x0092,
    DW_OP_piece                              = 0x0093,
    DW_OP_deref_size                         = 0x0094,
    DW_OP_xderef_size                        = 0x0095,
    DW_OP_nop                                = 0x0096,
    DW_OP_push_object_address                = 0x0097,
    DW_OP_call2                              = 0x0098,
    DW_OP_call4                              = 0x0099,
    DW_OP_call_ref                           = 0x009a,
    DW_OP_form_tls_address                   = 0x009b,
    DW_OP_call_frame_cfa                     = 0x009c,
    DW_OP_bit_piece                          = 0x009d,
    DW_OP_implicit_value                     = 0x009e,
    DW_OP_stack_value                        = 0x009f,
    DW_OP_implicit_pointer                   = 0x00a0,
    DW_OP_addrx                              = 0x00a1,
    DW_OP_constx                             = 0x00a2,
    DW_OP_entry_value                        = 0x00a3,
    DW_OP_const_type                         = 0x00a4,
    DW_OP_regval_type                        = 0x00a5,
    DW_OP_deref_type                         = 0x00a6,
    DW_OP_xderef_type                        = 0x00a7,
    DW_OP_convert                            = 0x00a8,
    DW_OP_reinterpret                        = 0x00a9,
    DW_OP_GNU_push_tls_address               = 0x00e0,
    DW_OP_HP_is_value                        = 0x00e1,
    DW_OP_HP_fltconst4                       = 0x00e2,
    DW_OP_HP_fltconst8                       = 0x00e3,
    DW_OP_HP_mod_range                       = 0x00e4,
    DW_OP_HP_unmod_range                     = 0x00e5,
    DW_OP_HP_tls                             = 0x00e6,
    DW_OP_INTEL_bit_piece                    = 0x00e8,
    DW_OP_GNU_uninit                         = 0x00f0,
    DW_OP_GNU_encoded_addr                   = 0x00f1,
    DW_OP_GNU_implicit_pointer               = 0x00f2,
    DW_OP_GNU_entry_value                    = 0x00f3,
    DW_OP_GNU_const_type                     = 0x00f4,
    DW_OP_GNU_regval_type                    = 0x00f5,
    DW_OP_GNU_deref_type                     = 0x00f6,
    DW_OP_GNU_convert                        = 0x00f7,
    DW_OP_PGI_omp_thread_num                 = 0x00f8,
    DW_OP_GNU_reinterpret                    = 0x00f9,
    DW_OP_GNU_parameter_ref                  = 0x00fa,
    DW_OP_GNU_addr_index                     = 0x00fb,
    DW_OP_GNU_const_index                    = 0x00fc,
    DW_OP_GNU_variable_value                 = 0x00fd,
    DW_OP_hi_user                            = 0x00ff
};

enum Dwarf_ATE_e {
    DW_ATE_address                           = 0x0001,
    DW_ATE_boolean                           = 0x0002,
    DW_ATE_complex_float                     = 0x0003,
    DW_ATE_float                             = 0x0004,
    DW_ATE_signed                            = 0x0005,
    DW_ATE_signed_char                       = 0x0006,
    DW_ATE_unsigned                          = 0x0007,
    DW_ATE_unsigned_char                     = 0x0008,
    DW_ATE_imaginary_float                   = 0x0009,
    DW_ATE_packed_decimal                    = 0x000a,
    DW_ATE_numeric_string                    = 0x000b,
    DW_ATE_edited                            = 0x000c,
    DW_ATE_signed_fixed                      = 0x000d,
    DW_ATE_unsigned_fixed                    = 0x000e,
    DW_ATE_decimal_float                     = 0x000f,
    DW_ATE_UTF                               = 0x0010,
    DW_ATE_UCS                               = 0x0011,
    DW_ATE_ASCII                             = 0x0012,
    DW_ATE_ALTIUM_fract                      = 0x0080,
    DW_ATE_ALTIUM_accum                      = 0x0081,
    DW_ATE_HP_float128                       = 0x0082,
    DW_ATE_HP_complex_float128               = 0x0083,
    DW_ATE_HP_floathpintel                   = 0x0084,
    DW_ATE_HP_imaginary_float80              = 0x0085,
    DW_ATE_HP_imaginary_float128             = 0x0086,
    DW_ATE_SUN_interval_float                = 0x0091,
    DW_ATE_SUN_imaginary_float               = 0x0092,
    DW_ATE_hi_user                           = 0x00ff
};

enum Dwarf_DEFAULTED_e {
    DW_DEFAULTED_no                          = 0x0000,
    DW_DEFAULTED_in_class                    = 0x0001,
    DW_DEFAULTED_out_of_class                = 0x0002
};

enum Dwarf_IDX_e {
    DW_IDX_compile_unit                      = 0x0001,
    DW_IDX_type_unit                         = 0x0002,
    DW_IDX_die_offset                        = 0x0003,
    DW_IDX_parent                            = 0x0004,
    DW_IDX_type_hash                         = 0x0005,
    DW_IDX_hi_user                           = 0x0fff,
    DW_IDX_lo_user                           = 0x2000
};

enum Dwarf_LLEX_e {
    DW_LLEX_end_of_list_entry                = 0x0000,
    DW_LLEX_base_address_selection_entry     = 0x0001,
    DW_LLEX_start_end_entry                  = 0x0002,
    DW_LLEX_start_length_entry               = 0x0003,
    DW_LLEX_offset_pair_entry                = 0x0004
};

enum Dwarf_LLE_e {
    DW_LLE_end_of_list                       = 0x0000,
    DW_LLE_base_addressx                     = 0x0001,
    DW_LLE_startx_endx                       = 0x0002,
    DW_LLE_startx_length                     = 0x0003,
    DW_LLE_offset_pair                       = 0x0004,
    DW_LLE_default_location                  = 0x0005,
    DW_LLE_base_address                      = 0x0006,
    DW_LLE_start_end                         = 0x0007,
    DW_LLE_start_length                      = 0x0008
};

enum Dwarf_RLE_e {
    DW_RLE_end_of_list                       = 0x0000,
    DW_RLE_base_addressx                     = 0x0001,
    DW_RLE_startx_endx                       = 0x0002,
    DW_RLE_startx_length                     = 0x0003,
    DW_RLE_offset_pair                       = 0x0004,
    DW_RLE_base_address                      = 0x0005,
    DW_RLE_start_end                         = 0x0006,
    DW_RLE_start_length                      = 0x0007
};

enum Dwarf_GNUIVIS_e {
    DW_GNUIVIS_global                        = 0x0000,
    DW_GNUIVIS_static                        = 0x0001
};

enum Dwarf_GNUIKIND_e {
    DW_GNUIKIND_none                         = 0x0000,
    DW_GNUIKIND_type                         = 0x0001,
    DW_GNUIKIND_variable                     = 0x0002,
    DW_GNUIKIND_function                     = 0x0003,
    DW_GNUIKIND_other                        = 0x0004
};

enum Dwarf_UT_e {
    DW_UT_compile                            = 0x0001,
    DW_UT_type                               = 0x0002,
    DW_UT_partial                            = 0x0003,
    DW_UT_skeleton                           = 0x0004,
    DW_UT_split_compile                      = 0x0005,
    DW_UT_split_type                         = 0x0006,
    DW_UT_lo_user                            = 0x0080,
    DW_UT_hi_user                            = 0x00ff
};

enum Dwarf_SECT_e {
    DW_SECT_INFO                             = 0x0001,
    DW_SECT_TYPES                            = 0x0002,
    DW_SECT_ABBREV                           = 0x0003,
    DW_SECT_LINE                             = 0x0004,
    DW_SECT_LOCLISTS                         = 0x0005,
    DW_SECT_STR_OFFSETS                      = 0x0006,
    DW_SECT_MACRO                            = 0x0007,
    DW_SECT_RNGLISTS                         = 0x0008
};

enum Dwarf_DS_e {
    DW_DS_unsigned                           = 0x0001,
    DW_DS_leading_overpunch                  = 0x0002,
    DW_DS_trailing_overpunch                 = 0x0003,
    DW_DS_leading_separate                   = 0x0004,
    DW_DS_trailing_separate                  = 0x0005
};

enum Dwarf_END_e {
    DW_END_default                           = 0x0000,
    DW_END_big                               = 0x0001,
    DW_END_little                            = 0x0002,
    DW_END_lo_user                           = 0x0040,
    DW_END_hi_user                           = 0x00ff
};

enum Dwarf_ATCF_e {
    DW_ATCF_lo_user                          = 0x0040,
    DW_ATCF_SUN_mop_bitfield                 = 0x0041,
    DW_ATCF_SUN_mop_spill                    = 0x0042,
    DW_ATCF_SUN_mop_scopy                    = 0x0043,
    DW_ATCF_SUN_func_start                   = 0x0044,
    DW_ATCF_SUN_end_ctors                    = 0x0045,
    DW_ATCF_SUN_branch_target                = 0x0046,
    DW_ATCF_SUN_mop_stack_probe              = 0x0047,
    DW_ATCF_SUN_func_epilog                  = 0x0048,
    DW_ATCF_hi_user                          = 0x00ff
};

enum Dwarf_ACCESS_e {
    DW_ACCESS_public                         = 0x0001,
    DW_ACCESS_protected                      = 0x0002,
    DW_ACCESS_private                        = 0x0003
};

enum Dwarf_VIS_e {
    DW_VIS_local                             = 0x0001,
    DW_VIS_exported                          = 0x0002,
    DW_VIS_qualified                         = 0x0003
};

enum Dwarf_VIRTUALITY_e {
    DW_VIRTUALITY_none                       = 0x0000,
    DW_VIRTUALITY_virtual                    = 0x0001,
    DW_VIRTUALITY_pure_virtual               = 0x0002
};

enum Dwarf_LANG_e {
    DW_LANG_C89                              = 0x0001,
    DW_LANG_C                                = 0x0002,
    DW_LANG_Ada83                            = 0x0003,
    DW_LANG_C_plus_plus                      = 0x0004,
    DW_LANG_Cobol74                          = 0x0005,
    DW_LANG_Cobol85                          = 0x0006,
    DW_LANG_Fortran77                        = 0x0007,
    DW_LANG_Fortran90                        = 0x0008,
    DW_LANG_Pascal83                         = 0x0009,
    DW_LANG_Modula2                          = 0x000a,
    DW_LANG_Java                             = 0x000b,
    DW_LANG_C99                              = 0x000c,
    DW_LANG_Ada95                            = 0x000d,
    DW_LANG_Fortran95                        = 0x000e,
    DW_LANG_PLI                              = 0x000f,
    DW_LANG_ObjC                             = 0x0010,
    DW_LANG_ObjC_plus_plus                   = 0x0011,
    DW_LANG_UPC                              = 0x0012,
    DW_LANG_D                                = 0x0013,
    DW_LANG_Python                           = 0x0014,
    DW_LANG_OpenCL                           = 0x0015,
    DW_LANG_Go                               = 0x0016,
    DW_LANG_Modula3                          = 0x0017,
    DW_LANG_Haskel                           = 0x0018,
    DW_LANG_C_plus_plus_03                   = 0x0019,
    DW_LANG_C_plus_plus_11                   = 0x001a,
    DW_LANG_OCaml                            = 0x001b,
    DW_LANG_Rust                             = 0x001c,
    DW_LANG_C11                              = 0x001d,
    DW_LANG_Swift                            = 0x001e,
    DW_LANG_Julia                            = 0x001f,
    DW_LANG_Dylan                            = 0x0020,
    DW_LANG_C_plus_plus_14                   = 0x0021,
    DW_LANG_Fortran03                        = 0x0022,
    DW_LANG_Fortran08                        = 0x0023,
    DW_LANG_RenderScript                     = 0x0024,
    DW_LANG_BLISS                            = 0x0025,
    DW_LANG_lo_user                          = 0x8000,
    DW_LANG_Mips_Assembler                   = 0x8001,
    DW_LANG_Upc                              = 0x8765,
    DW_LANG_SUN_Assembler                    = 0x9001,
    DW_LANG_ALTIUM_Assembler                 = 0x9101,
    DW_LANG_hi_user                          = 0xffff
};

enum Dwarf_ID_e {
    DW_ID_case_sensitive                     = 0x0000,
    DW_ID_up_case                            = 0x0001,
    DW_ID_down_case                          = 0x0002,
    DW_ID_case_insensitive                   = 0x0003
};

enum Dwarf_CC_e {
    DW_CC_normal                             = 0x0001,
    DW_CC_program                            = 0x0002,
    DW_CC_nocall                             = 0x0003,
    DW_CC_pass_by_reference                  = 0x0004,
    DW_CC_pass_by_value                      = 0x0005,
    DW_CC_lo_user                            = 0x0040,
    DW_CC_GNU_borland_fastcall_i386          = 0x0041,
    DW_CC_ALTIUM_interrupt                   = 0x0065,
    DW_CC_ALTIUM_near_system_stack           = 0x0066,
    DW_CC_ALTIUM_near_user_stack             = 0x0067,
    DW_CC_ALTIUM_huge_user_stack             = 0x0068,
    DW_CC_hi_user                            = 0x00ff
};

enum Dwarf_INL_e {
    DW_INL_not_inlined                       = 0x0000,
    DW_INL_inlined                           = 0x0001,
    DW_INL_declared_not_inlined              = 0x0002,
    DW_INL_declared_inlined                  = 0x0003
};

enum Dwarf_ORD_e {
    DW_ORD_row_major                         = 0x0000,
    DW_ORD_col_major                         = 0x0001
};

enum Dwarf_DSC_e {
    DW_DSC_label                             = 0x0000,
    DW_DSC_range                             = 0x0001
};

enum Dwarf_LNCT_e {
    DW_LNCT_path                             = 0x0001,
    DW_LNCT_directory_index                  = 0x0002,
    DW_LNCT_timestamp                        = 0x0003,
    DW_LNCT_size                             = 0x0004,
    DW_LNCT_MD5                              = 0x0005,
    DW_LNCT_GNU_subprogram_name              = 0x0006,
    DW_LNCT_GNU_decl_file                    = 0x0007,
    DW_LNCT_GNU_decl_line                    = 0x0008,
    DW_LNCT_lo_user                          = 0x2000,
    DW_LNCT_hi_user                          = 0x3fff
};

enum Dwarf_LNS_e {
    DW_LNS_copy                              = 0x0001,
    DW_LNS_advance_pc                        = 0x0002,
    DW_LNS_advance_line                      = 0x0003,
    DW_LNS_set_file                          = 0x0004,
    DW_LNS_set_column                        = 0x0005,
    DW_LNS_negate_stmt                       = 0x0006,
    DW_LNS_set_basic_block                   = 0x0007,
    DW_LNS_const_add_pc                      = 0x0008,
    DW_LNS_fixed_advance_pc                  = 0x0009,
    DW_LNS_set_prologue_end                  = 0x000a,
    DW_LNS_set_epilogue_begin                = 0x000b,
    DW_LNS_set_isa                           = 0x000c,
    DW_LNS_set_address_from_logical          = 0x000d,
    DW_LNS_inlined_call                      = 0x000e,
    DW_LNS_pop_context                       = 0x000f
};

enum Dwarf_LNE_e {
    DW_LNE_end_sequence                      = 0x0001,
    DW_LNE_set_address                       = 0x0002,
    DW_LNE_define_file                       = 0x0003,
    DW_LNE_set_discriminator                 = 0x0004,
    DW_LNE_HP_negate_is_UV_update            = 0x0011,
    DW_LNE_HP_push_context                   = 0x0012,
    DW_LNE_HP_pop_context                    = 0x0013,
    DW_LNE_HP_set_file_line_column           = 0x0014,
    DW_LNE_HP_set_routine_name               = 0x0015,
    DW_LNE_HP_set_sequence                   = 0x0016,
    DW_LNE_HP_negate_post_semantics          = 0x0017,
    DW_LNE_HP_negate_function_exit           = 0x0018,
    DW_LNE_HP_negate_front_end_logical       = 0x0019,
    DW_LNE_HP_define_proc                    = 0x0020,
    DW_LNE_HP_source_file_correlation        = 0x0080,
    DW_LNE_hi_user                           = 0x00ff
};

enum Dwarf_ISA_e {
    DW_ISA_UNKNOWN                           = 0x0000,
    DW_ISA_ARM_thumb                         = 0x0001,
    DW_ISA_ARM_arm                           = 0x0002
};

enum Dwarf_MACRO_e {
    DW_MACRO_define                          = 0x0001,
    DW_MACRO_undef                           = 0x0002,
    DW_MACRO_start_file                      = 0x0003,
    DW_MACRO_end_file                        = 0x0004,
    DW_MACRO_define_strp                     = 0x0005,
    DW_MACRO_undef_strp                      = 0x0006,
    DW_MACRO_import                          = 0x0007,
    DW_MACRO_define_sup                      = 0x0008,
    DW_MACRO_undef_sup                       = 0x0009,
    DW_MACRO_import_sup                      = 0x000a,
    DW_MACRO_define_strx                     = 0x000b,
    DW_MACRO_undef_strx                      = 0x000c,
    DW_MACRO_lo_user                         = 0x00e0,
    DW_MACRO_hi_user                         = 0x00ff
};

enum Dwarf_MACINFO_e {
    DW_MACINFO_define                        = 0x0001,
    DW_MACINFO_undef                         = 0x0002,
    DW_MACINFO_start_file                    = 0x0003,
    DW_MACINFO_end_file                      = 0x0004,
    DW_MACINFO_vendor_ext                    = 0x00ff
};

enum Dwarf_CFA_e {
    DW_CFA_extended                          = 0x0000,
    DW_CFA_set_loc                           = 0x0001,
    DW_CFA_advance_loc1                      = 0x0002,
    DW_CFA_advance_loc2                      = 0x0003,
    DW_CFA_advance_loc4                      = 0x0004,
    DW_CFA_offset_extended                   = 0x0005,
    DW_CFA_restore_extended                  = 0x0006,
    DW_CFA_undefined                         = 0x0007,
    DW_CFA_same_value                        = 0x0008,
    DW_CFA_register                          = 0x0009,
    DW_CFA_remember_state                    = 0x000a,
    DW_CFA_restore_state                     = 0x000b,
    DW_CFA_def_cfa                           = 0x000c,
    DW_CFA_def_cfa_register                  = 0x000d,
    DW_CFA_def_cfa_offset                    = 0x000e,
    DW_CFA_def_cfa_expression                = 0x000f,
    DW_CFA_expression                        = 0x0010,
    DW_CFA_offset_extended_sf                = 0x0011,
    DW_CFA_def_cfa_sf                        = 0x0012,
    DW_CFA_def_cfa_offset_sf                 = 0x0013,
    DW_CFA_val_offset                        = 0x0014,
    DW_CFA_val_offset_sf                     = 0x0015,
    DW_CFA_val_expression                    = 0x0016,
    DW_CFA_lo_user                           = 0x001c,
    DW_CFA_MIPS_advance_loc8                 = 0x001d,
    DW_CFA_GNU_window_save                   = 0x002d,
    DW_CFA_GNU_args_size                     = 0x002e,
    DW_CFA_GNU_negative_offset_extended      = 0x002f,
    DW_CFA_METAWARE_info                     = 0x0034,
    DW_CFA_high_user                         = 0x003f,
    DW_CFA_advance_loc                       = 0x0040,
    DW_CFA_offset                            = 0x0080,
    DW_CFA_restore                           = 0x00c0
};

enum Dwarf_EH_e {
    DW_EH_PE_absptr                          = 0x0000,
    DW_EH_PE_uleb128                         = 0x0001,
    DW_EH_PE_udata2                          = 0x0002,
    DW_EH_PE_udata4                          = 0x0003,
    DW_EH_PE_udata8                          = 0x0004,
    DW_EH_PE_sleb128                         = 0x0009,
    DW_EH_PE_sdata2                          = 0x000a,
    DW_EH_PE_sdata4                          = 0x000b,
    DW_EH_PE_sdata8                          = 0x000c,
    DW_EH_PE_pcrel                           = 0x0010,
    DW_EH_PE_textrel                         = 0x0020,
    DW_EH_PE_datarel                         = 0x0030,
    DW_EH_PE_funcrel                         = 0x0040,
    DW_EH_PE_aligned                         = 0x0050,
    DW_EH_PE_omit                            = 0x00ff
};

enum Dwarf_FRAME_e {
    DW_FRAME_CFA_COL                         = 0x0000,
    DW_FRAME_REG1                            = 0x0001,
    DW_FRAME_REG2                            = 0x0002,
    DW_FRAME_REG3                            = 0x0003,
    DW_FRAME_REG4                            = 0x0004,
    DW_FRAME_REG5                            = 0x0005,
    DW_FRAME_REG6                            = 0x0006,
    DW_FRAME_REG7                            = 0x0007,
    DW_FRAME_REG8                            = 0x0008,
    DW_FRAME_REG9                            = 0x0009,
    DW_FRAME_REG10                           = 0x0010,
    DW_FRAME_REG11                           = 0x0011,
    DW_FRAME_REG12                           = 0x0012,
    DW_FRAME_REG13                           = 0x0013,
    DW_FRAME_REG14                           = 0x0014,
    DW_FRAME_REG15                           = 0x0015,
    DW_FRAME_REG16                           = 0x0016,
    DW_FRAME_REG17                           = 0x0017,
    DW_FRAME_REG18                           = 0x0018,
    DW_FRAME_REG19                           = 0x0019,
    DW_FRAME_REG20                           = 0x0020,
    DW_FRAME_REG21                           = 0x0021,
    DW_FRAME_REG22                           = 0x0022,
    DW_FRAME_REG23                           = 0x0023,
    DW_FRAME_REG24                           = 0x0024,
    DW_FRAME_REG25                           = 0x0025,
    DW_FRAME_REG26                           = 0x0026,
    DW_FRAME_REG27                           = 0x0027,
    DW_FRAME_REG28                           = 0x0028,
    DW_FRAME_REG29                           = 0x0029,
    DW_FRAME_REG30                           = 0x0030,
    DW_FRAME_REG31                           = 0x0031,
    DW_FRAME_FREG0                           = 0x0032,
    DW_FRAME_FREG1                           = 0x0033,
    DW_FRAME_FREG2                           = 0x0034,
    DW_FRAME_FREG3                           = 0x0035,
    DW_FRAME_FREG4                           = 0x0036,
    DW_FRAME_FREG5                           = 0x0037,
    DW_FRAME_FREG6                           = 0x0038,
    DW_FRAME_FREG7                           = 0x0039,
    DW_FRAME_FREG8                           = 0x0040,
    DW_FRAME_FREG9                           = 0x0041,
    DW_FRAME_FREG10                          = 0x0042,
    DW_FRAME_FREG11                          = 0x0043,
    DW_FRAME_FREG12                          = 0x0044,
    DW_FRAME_FREG13                          = 0x0045,
    DW_FRAME_FREG14                          = 0x0046,
    DW_FRAME_FREG15                          = 0x0047,
    DW_FRAME_FREG16                          = 0x0048,
    DW_FRAME_FREG17                          = 0x0049,
    DW_FRAME_FREG18                          = 0x0050,
    DW_FRAME_FREG19                          = 0x0051,
    DW_FRAME_FREG20                          = 0x0052,
    DW_FRAME_FREG21                          = 0x0053,
    DW_FRAME_FREG22                          = 0x0054,
    DW_FRAME_FREG23                          = 0x0055,
    DW_FRAME_FREG24                          = 0x0056,
    DW_FRAME_FREG25                          = 0x0057,
    DW_FRAME_FREG26                          = 0x0058,
    DW_FRAME_FREG27                          = 0x0059,
    DW_FRAME_FREG28                          = 0x0060,
    DW_FRAME_FREG29                          = 0x0061,
    DW_FRAME_FREG30                          = 0x0062,
    DW_FRAME_FREG31                          = 0x0063,
    DW_FRAME_FREG32                          = 0x0064,
    DW_FRAME_FREG33                          = 0x0065,
    DW_FRAME_FREG34                          = 0x0066,
    DW_FRAME_FREG35                          = 0x0067,
    DW_FRAME_FREG36                          = 0x0068,
    DW_FRAME_FREG37                          = 0x0069,
    DW_FRAME_FREG38                          = 0x0070,
    DW_FRAME_FREG39                          = 0x0071,
    DW_FRAME_FREG40                          = 0x0072,
    DW_FRAME_FREG41                          = 0x0073,
    DW_FRAME_FREG42                          = 0x0074,
    DW_FRAME_FREG43                          = 0x0075,
    DW_FRAME_FREG44                          = 0x0076,
    DW_FRAME_FREG45                          = 0x0077,
    DW_FRAME_FREG46                          = 0x0078,
    DW_FRAME_FREG47                          = 0x0079,
    DW_FRAME_FREG48                          = 0x0080,
    DW_FRAME_FREG49                          = 0x0081,
    DW_FRAME_FREG50                          = 0x0082,
    DW_FRAME_FREG51                          = 0x0083,
    DW_FRAME_FREG52                          = 0x0084,
    DW_FRAME_FREG53                          = 0x0085,
    DW_FRAME_FREG54                          = 0x0086,
    DW_FRAME_FREG55                          = 0x0087,
    DW_FRAME_FREG56                          = 0x0088,
    DW_FRAME_FREG57                          = 0x0089,
    DW_FRAME_FREG58                          = 0x0090,
    DW_FRAME_FREG59                          = 0x0091,
    DW_FRAME_FREG60                          = 0x0092,
    DW_FRAME_FREG61                          = 0x0093,
    DW_FRAME_FREG62                          = 0x0094,
    DW_FRAME_FREG63                          = 0x0095,
    DW_FRAME_FREG64                          = 0x0096,
    DW_FRAME_FREG65                          = 0x0097,
    DW_FRAME_FREG66                          = 0x0098,
    DW_FRAME_FREG67                          = 0x0099,
    DW_FRAME_FREG68                          = 0x0100,
    DW_FRAME_FREG69                          = 0x0101,
    DW_FRAME_FREG70                          = 0x0102,
    DW_FRAME_FREG71                          = 0x0103,
    DW_FRAME_FREG72                          = 0x0104,
    DW_FRAME_FREG73                          = 0x0105,
    DW_FRAME_FREG74                          = 0x0106,
    DW_FRAME_FREG75                          = 0x0107,
    DW_FRAME_FREG76                          = 0x0108,
    DW_FRAME_HIGHEST_NORMAL_REGISTER         = 0x0188
};

enum Dwarf_CHILDREN_e {
    DW_CHILDREN_no                           = 0x0000,
    DW_CHILDREN_yes                          = 0x0001
};

enum Dwarf_ADDR_e {
    DW_ADDR_none                             = 0x0000
};
#endif /* __DWARF_NAMES_ENUM_H__ */

/* END FILE */
