/* Created by build_access.py */
/* returns string of length 0 if invalid arg */
const char * dwarf_get_elf_relocname_ppc64(unsigned long);
#ifndef R_PPC64_ADDR30
#define R_PPC64_ADDR30       37
#endif /* R_PPC64_ADDR30 */
#ifndef R_PPC64_ADDR64
#define R_PPC64_ADDR64       38
#endif /* R_PPC64_ADDR64 */
#ifndef R_PPC64_ADDR16_HIGHER
#define R_PPC64_ADDR16_HIGHER 39
#endif /* R_PPC64_ADDR16_HIGHER */
#ifndef R_PPC64_ADDR16_HIGHERA
#define R_PPC64_ADDR16_HIGHERA 40
#endif /* R_PPC64_ADDR16_HIGHERA */
#ifndef R_PPC64_ADDR16_HIGHEST
#define R_PPC64_ADDR16_HIGHEST 41
#endif /* R_PPC64_ADDR16_HIGHEST */
#ifndef R_PPC64_ADDR16_HIGHESTA
#define R_PPC64_ADDR16_HIGHESTA 42
#endif /* R_PPC64_ADDR16_HIGHESTA */
#ifndef R_PPC64_UADDR64
#define R_PPC64_UADDR64      43
#endif /* R_PPC64_UADDR64 */
#ifndef R_PPC64_REL64
#define R_PPC64_REL64        44
#endif /* R_PPC64_REL64 */
#ifndef R_PPC64_PLT64
#define R_PPC64_PLT64        45
#endif /* R_PPC64_PLT64 */
#ifndef R_PPC64_PLTREL64
#define R_PPC64_PLTREL64     46
#endif /* R_PPC64_PLTREL64 */
#ifndef R_PPC64_TOC16
#define R_PPC64_TOC16        47
#endif /* R_PPC64_TOC16 */
#ifndef R_PPC64_TOC16_LO
#define R_PPC64_TOC16_LO     48
#endif /* R_PPC64_TOC16_LO */
#ifndef R_PPC64_TOC16_HI
#define R_PPC64_TOC16_HI     49
#endif /* R_PPC64_TOC16_HI */
#ifndef R_PPC64_TOC16_HA
#define R_PPC64_TOC16_HA     50
#endif /* R_PPC64_TOC16_HA */
#ifndef R_PPC64_TOC
#define R_PPC64_TOC          51
#endif /* R_PPC64_TOC */
#ifndef R_PPC64_PLTGOT16
#define R_PPC64_PLTGOT16     52
#endif /* R_PPC64_PLTGOT16 */
#ifndef R_PPC64_PLTGOT16_LO
#define R_PPC64_PLTGOT16_LO  53
#endif /* R_PPC64_PLTGOT16_LO */
#ifndef R_PPC64_PLTGOT16_HI
#define R_PPC64_PLTGOT16_HI  54
#endif /* R_PPC64_PLTGOT16_HI */
#ifndef R_PPC64_PLTGOT16_HA
#define R_PPC64_PLTGOT16_HA  55
#endif /* R_PPC64_PLTGOT16_HA */
#ifndef R_PPC64_ADDR16_DS
#define R_PPC64_ADDR16_DS    56
#endif /* R_PPC64_ADDR16_DS */
#ifndef R_PPC64_ADDR16_LO_DS
#define R_PPC64_ADDR16_LO_DS 57
#endif /* R_PPC64_ADDR16_LO_DS */
#ifndef R_PPC64_GOT16_DS
#define R_PPC64_GOT16_DS     58
#endif /* R_PPC64_GOT16_DS */
#ifndef R_PPC64_GOT16_LO_DS
#define R_PPC64_GOT16_LO_DS  59
#endif /* R_PPC64_GOT16_LO_DS */
#ifndef R_PPC64_PLT16_LO_DS
#define R_PPC64_PLT16_LO_DS  60
#endif /* R_PPC64_PLT16_LO_DS */
#ifndef R_PPC64_SECTOFF_DS
#define R_PPC64_SECTOFF_DS   61
#endif /* R_PPC64_SECTOFF_DS */
#ifndef R_PPC64_SECTOFF_LO_DS
#define R_PPC64_SECTOFF_LO_DS 62
#endif /* R_PPC64_SECTOFF_LO_DS */
#ifndef R_PPC64_TOC16_DS
#define R_PPC64_TOC16_DS     63
#endif /* R_PPC64_TOC16_DS */
#ifndef R_PPC64_TOC16_LO_DS
#define R_PPC64_TOC16_LO_DS  64
#endif /* R_PPC64_TOC16_LO_DS */
#ifndef R_PPC64_PLTGOT16_DS
#define R_PPC64_PLTGOT16_DS  65
#endif /* R_PPC64_PLTGOT16_DS */
#ifndef R_PPC64_PLTGOT16_LO_DS
#define R_PPC64_PLTGOT16_LO_DS 66
#endif /* R_PPC64_PLTGOT16_LO_DS */
#ifndef R_PPC64_TLS
#define R_PPC64_TLS          67
#endif /* R_PPC64_TLS */
#ifndef R_PPC64_DTPMOD64
#define R_PPC64_DTPMOD64     68
#endif /* R_PPC64_DTPMOD64 */
#ifndef R_PPC64_TPREL16
#define R_PPC64_TPREL16      69
#endif /* R_PPC64_TPREL16 */
#ifndef R_PPC64_TPREL16_LO
#define R_PPC64_TPREL16_LO   70
#endif /* R_PPC64_TPREL16_LO */
#ifndef R_PPC64_TPREL16_HI
#define R_PPC64_TPREL16_HI   71
#endif /* R_PPC64_TPREL16_HI */
#ifndef R_PPC64_TPREL16_HA
#define R_PPC64_TPREL16_HA   72
#endif /* R_PPC64_TPREL16_HA */
#ifndef R_PPC64_TPREL64
#define R_PPC64_TPREL64      73
#endif /* R_PPC64_TPREL64 */
#ifndef R_PPC64_DTPREL16
#define R_PPC64_DTPREL16     74
#endif /* R_PPC64_DTPREL16 */
#ifndef R_PPC64_DTPREL16_LO
#define R_PPC64_DTPREL16_LO  75
#endif /* R_PPC64_DTPREL16_LO */
#ifndef R_PPC64_DTPREL16_HI
#define R_PPC64_DTPREL16_HI  76
#endif /* R_PPC64_DTPREL16_HI */
#ifndef R_PPC64_DTPREL16_HA
#define R_PPC64_DTPREL16_HA  77
#endif /* R_PPC64_DTPREL16_HA */
#ifndef R_PPC64_DTPREL64
#define R_PPC64_DTPREL64     78
#endif /* R_PPC64_DTPREL64 */
#ifndef R_PPC64_GOT_TLSGD16
#define R_PPC64_GOT_TLSGD16  79
#endif /* R_PPC64_GOT_TLSGD16 */
#ifndef R_PPC64_GOT_TLSGD16_LO
#define R_PPC64_GOT_TLSGD16_LO 80
#endif /* R_PPC64_GOT_TLSGD16_LO */
#ifndef R_PPC64_GOT_TLSGD16_HI
#define R_PPC64_GOT_TLSGD16_HI 81
#endif /* R_PPC64_GOT_TLSGD16_HI */
#ifndef R_PPC64_GOT_TLSGD16_HA
#define R_PPC64_GOT_TLSGD16_HA 82
#endif /* R_PPC64_GOT_TLSGD16_HA */
#ifndef R_PPC64_GOT_TLSLD16
#define R_PPC64_GOT_TLSLD16  83
#endif /* R_PPC64_GOT_TLSLD16 */
#ifndef R_PPC64_GOT_TLSLD16_LO
#define R_PPC64_GOT_TLSLD16_LO 84
#endif /* R_PPC64_GOT_TLSLD16_LO */
#ifndef R_PPC64_GOT_TLSLD16_HI
#define R_PPC64_GOT_TLSLD16_HI 85
#endif /* R_PPC64_GOT_TLSLD16_HI */
#ifndef R_PPC64_GOT_TLSLD16_HA
#define R_PPC64_GOT_TLSLD16_HA 86
#endif /* R_PPC64_GOT_TLSLD16_HA */
#ifndef R_PPC64_GOT_TPREL16_DS
#define R_PPC64_GOT_TPREL16_DS 87
#endif /* R_PPC64_GOT_TPREL16_DS */
#ifndef R_PPC64_GOT_TPREL16_LO_DS
#define R_PPC64_GOT_TPREL16_LO_DS 88
#endif /* R_PPC64_GOT_TPREL16_LO_DS */
#ifndef R_PPC64_GOT_TPREL16_HI
#define R_PPC64_GOT_TPREL16_HI 89
#endif /* R_PPC64_GOT_TPREL16_HI */
#ifndef R_PPC64_GOT_TPREL16_HA
#define R_PPC64_GOT_TPREL16_HA 90
#endif /* R_PPC64_GOT_TPREL16_HA */
#ifndef R_PPC64_GOT_DTPREL16_DS
#define R_PPC64_GOT_DTPREL16_DS 91
#endif /* R_PPC64_GOT_DTPREL16_DS */
#ifndef R_PPC64_GOT_DTPREL16_LO_DS
#define R_PPC64_GOT_DTPREL16_LO_DS 92
#endif /* R_PPC64_GOT_DTPREL16_LO_DS */
#ifndef R_PPC64_GOT_DTPREL16_HI
#define R_PPC64_GOT_DTPREL16_HI 93
#endif /* R_PPC64_GOT_DTPREL16_HI */
#ifndef R_PPC64_GOT_DTPREL16_HA
#define R_PPC64_GOT_DTPREL16_HA 94
#endif /* R_PPC64_GOT_DTPREL16_HA */
#ifndef R_PPC64_TPREL16_DS
#define R_PPC64_TPREL16_DS   95
#endif /* R_PPC64_TPREL16_DS */
#ifndef R_PPC64_TPREL16_LO_DS
#define R_PPC64_TPREL16_LO_DS 96
#endif /* R_PPC64_TPREL16_LO_DS */
#ifndef R_PPC64_TPREL16_HIGHER
#define R_PPC64_TPREL16_HIGHER 97
#endif /* R_PPC64_TPREL16_HIGHER */
#ifndef R_PPC64_TPREL16_HIGHERA
#define R_PPC64_TPREL16_HIGHERA 98
#endif /* R_PPC64_TPREL16_HIGHERA */
#ifndef R_PPC64_TPREL16_HIGHEST
#define R_PPC64_TPREL16_HIGHEST 99
#endif /* R_PPC64_TPREL16_HIGHEST */
#ifndef R_PPC64_TPREL16_HIGHESTA
#define R_PPC64_TPREL16_HIGHESTA 100
#endif /* R_PPC64_TPREL16_HIGHESTA */
#ifndef R_PPC64_DTPREL16_DS
#define R_PPC64_DTPREL16_DS  101
#endif /* R_PPC64_DTPREL16_DS */
#ifndef R_PPC64_DTPREL16_LO_DS
#define R_PPC64_DTPREL16_LO_DS 102
#endif /* R_PPC64_DTPREL16_LO_DS */
#ifndef R_PPC64_DTPREL16_HIGHER
#define R_PPC64_DTPREL16_HIGHER 103
#endif /* R_PPC64_DTPREL16_HIGHER */
#ifndef R_PPC64_DTPREL16_HIGHERA
#define R_PPC64_DTPREL16_HIGHERA 104
#endif /* R_PPC64_DTPREL16_HIGHERA */
#ifndef R_PPC64_DTPREL16_HIGHEST
#define R_PPC64_DTPREL16_HIGHEST 105
#endif /* R_PPC64_DTPREL16_HIGHEST */
#ifndef R_PPC64_DTPREL16_HIGHESTA
#define R_PPC64_DTPREL16_HIGHESTA 106
#endif /* R_PPC64_DTPREL16_HIGHESTA */
#ifndef R_PPC64_TOC32
#define R_PPC64_TOC32        107
#endif /* R_PPC64_TOC32 */
#ifndef R_PPC64_DTPMOD32
#define R_PPC64_DTPMOD32     108
#endif /* R_PPC64_DTPMOD32 */
#ifndef R_PPC64_TPREL32
#define R_PPC64_TPREL32      109
#endif /* R_PPC64_TPREL32 */
#ifndef R_PPC64_DTPREL32
#define R_PPC64_DTPREL32     110
#endif /* R_PPC64_DTPREL32 */
#ifndef R_PPC64_ADDR16_HIGHA
#define R_PPC64_ADDR16_HIGHA 111
#endif /* R_PPC64_ADDR16_HIGHA */
#ifndef R_PPC64_TPREL16_HIGH
#define R_PPC64_TPREL16_HIGH 112
#endif /* R_PPC64_TPREL16_HIGH */
#ifndef R_PPC64_TPREL16_HIGHA
#define R_PPC64_TPREL16_HIGHA 113
#endif /* R_PPC64_TPREL16_HIGHA */
#ifndef R_PPC64_DTPREL16_HIGH
#define R_PPC64_DTPREL16_HIGH 114
#endif /* R_PPC64_DTPREL16_HIGH */
#ifndef R_PPC64_DTPREL16_HIGHA
#define R_PPC64_DTPREL16_HIGHA 115
#endif /* R_PPC64_DTPREL16_HIGHA */
#ifndef R_PPC64_JMP_IREL
#define R_PPC64_JMP_IREL     247
#endif /* R_PPC64_JMP_IREL */
#ifndef R_PPC64_IRELATIVE
#define R_PPC64_IRELATIVE    248
#endif /* R_PPC64_IRELATIVE */
#ifndef R_PPC64_REL16
#define R_PPC64_REL16        249
#endif /* R_PPC64_REL16 */
#ifndef R_PPC64_REL16_LO
#define R_PPC64_REL16_LO     250
#endif /* R_PPC64_REL16_LO */
#ifndef R_PPC64_REL16_HI
#define R_PPC64_REL16_HI     251
#endif /* R_PPC64_REL16_HI */
#ifndef R_PPC64_REL16_HA
#define R_PPC64_REL16_HA     252
#endif /* R_PPC64_REL16_HA */
