/* Created by build_access.py */
/* returns string of length 0 if invalid arg */
const char * dwarf_get_elf_relocname_386(unsigned long);
#ifndef R_386_NONE
#define R_386_NONE           0
#endif /* R_386_NONE */
#ifndef R_386_32
#define R_386_32             1
#endif /* R_386_32 */
#ifndef R_386_PC32
#define R_386_PC32           2
#endif /* R_386_PC32 */
#ifndef R_386_GOT32
#define R_386_GOT32          3
#endif /* R_386_GOT32 */
#ifndef R_386_PLT32
#define R_386_PLT32          4
#endif /* R_386_PLT32 */
#ifndef R_386_COPY
#define R_386_COPY           5
#endif /* R_386_COPY */
#ifndef R_386_GLOB_DAT
#define R_386_GLOB_DAT       6
#endif /* R_386_GLOB_DAT */
#ifndef R_386_JMP_SLOT
#define R_386_JMP_SLOT       7
#endif /* R_386_JMP_SLOT */
#ifndef R_386_RELATIVE
#define R_386_RELATIVE       8
#endif /* R_386_RELATIVE */
#ifndef R_386_GOTOFF
#define R_386_GOTOFF         9
#endif /* R_386_GOTOFF */
#ifndef R_386_GOTPC
#define R_386_GOTPC          10
#endif /* R_386_GOTPC */
#ifndef R_386_32PLT
#define R_386_32PLT          11
#endif /* R_386_32PLT */
#ifndef R_386_TLS_TPOFF
#define R_386_TLS_TPOFF      14
#endif /* R_386_TLS_TPOFF */
#ifndef R_386_TLS_IE
#define R_386_TLS_IE         15
#endif /* R_386_TLS_IE */
#ifndef R_386_TLS_GOTIE
#define R_386_TLS_GOTIE      16
#endif /* R_386_TLS_GOTIE */
#ifndef R_386_TLS_LE
#define R_386_TLS_LE         17
#endif /* R_386_TLS_LE */
#ifndef R_386_TLS_LDM
#define R_386_TLS_LDM        19
#endif /* R_386_TLS_LDM */
#ifndef R_386_16
#define R_386_16             20
#endif /* R_386_16 */
#ifndef R_386_PC16
#define R_386_PC16           21
#endif /* R_386_PC16 */
#ifndef R_386_8
#define R_386_8              22
#endif /* R_386_8 */
#ifndef R_386_PC8
#define R_386_PC8            23
#endif /* R_386_PC8 */
#ifndef R_386_TLS_GD_32
#define R_386_TLS_GD_32      24
#endif /* R_386_TLS_GD_32 */
#ifndef R_386_TLS_GD_PUSH
#define R_386_TLS_GD_PUSH    25
#endif /* R_386_TLS_GD_PUSH */
#ifndef R_386_TLS_GD_CALL
#define R_386_TLS_GD_CALL    26
#endif /* R_386_TLS_GD_CALL */
#ifndef R_386_TLS_GD_POP
#define R_386_TLS_GD_POP     27
#endif /* R_386_TLS_GD_POP */
#ifndef R_386_TLS_LDM_32
#define R_386_TLS_LDM_32     28
#endif /* R_386_TLS_LDM_32 */
#ifndef R_386_TLS_LDM_PUSH
#define R_386_TLS_LDM_PUSH   29
#endif /* R_386_TLS_LDM_PUSH */
#ifndef R_386_TLS_LDM_CALL
#define R_386_TLS_LDM_CALL   30
#endif /* R_386_TLS_LDM_CALL */
#ifndef R_386_TLS_LDM_POP
#define R_386_TLS_LDM_POP    31
#endif /* R_386_TLS_LDM_POP */
#ifndef R_386_TLS_LDO_32
#define R_386_TLS_LDO_32     32
#endif /* R_386_TLS_LDO_32 */
#ifndef R_386_TLS_IE_32
#define R_386_TLS_IE_32      33
#endif /* R_386_TLS_IE_32 */
#ifndef R_386_TLS_LE_32
#define R_386_TLS_LE_32      34
#endif /* R_386_TLS_LE_32 */
#ifndef R_386_TLS_DTPMOD32
#define R_386_TLS_DTPMOD32   35
#endif /* R_386_TLS_DTPMOD32 */
#ifndef R_386_TLS_DTPOFF32
#define R_386_TLS_DTPOFF32   36
#endif /* R_386_TLS_DTPOFF32 */
#ifndef R_386_TLS_TPOFF32
#define R_386_TLS_TPOFF32    37
#endif /* R_386_TLS_TPOFF32 */
#ifndef R_386_SIZE32
#define R_386_SIZE32         38
#endif /* R_386_SIZE32 */
#ifndef R_386_TLS_GOTDESC
#define R_386_TLS_GOTDESC    39
#endif /* R_386_TLS_GOTDESC */
#ifndef R_386_TLS_DESC_CALL
#define R_386_TLS_DESC_CALL  40
#endif /* R_386_TLS_DESC_CALL */
#ifndef R_386_TLS_DESC
#define R_386_TLS_DESC       41
#endif /* R_386_TLS_DESC */
#ifndef R_386_IRELATIVE
#define R_386_IRELATIVE      42
#endif /* R_386_IRELATIVE */
