/* Created by build_access.py */
/* returns string of length 0 if invalid arg */
const char * dwarf_get_elf_relocname_aarch64(unsigned long);
#ifndef R_AARCH64_NONE
#define R_AARCH64_NONE       0
#endif /* R_AARCH64_NONE */
#ifndef R_AARCH64_P32_ABS32
#define R_AARCH64_P32_ABS32  1
#endif /* R_AARCH64_P32_ABS32 */
#ifndef R_AARCH64_P32_COPY
#define R_AARCH64_P32_COPY   180
#endif /* R_AARCH64_P32_COPY */
#ifndef R_AARCH64_P32_GLOB_DAT
#define R_AARCH64_P32_GLOB_DAT 181
#endif /* R_AARCH64_P32_GLOB_DAT */
#ifndef R_AARCH64_P32_JUMP_SLOT
#define R_AARCH64_P32_JUMP_SLOT 182
#endif /* R_AARCH64_P32_JUMP_SLOT */
#ifndef R_AARCH64_P32_RELATIVE
#define R_AARCH64_P32_RELATIVE 183
#endif /* R_AARCH64_P32_RELATIVE */
#ifndef R_AARCH64_P32_TLS_DTPMOD
#define R_AARCH64_P32_TLS_DTPMOD 184
#endif /* R_AARCH64_P32_TLS_DTPMOD */
#ifndef R_AARCH64_P32_TLS_DTPREL
#define R_AARCH64_P32_TLS_DTPREL 185
#endif /* R_AARCH64_P32_TLS_DTPREL */
#ifndef R_AARCH64_P32_TLS_TPREL
#define R_AARCH64_P32_TLS_TPREL 186
#endif /* R_AARCH64_P32_TLS_TPREL */
#ifndef R_AARCH64_P32_TLSDESC
#define R_AARCH64_P32_TLSDESC 187
#endif /* R_AARCH64_P32_TLSDESC */
#ifndef R_AARCH64_P32_IRELATIVE
#define R_AARCH64_P32_IRELATIVE 188
#endif /* R_AARCH64_P32_IRELATIVE */
#ifndef R_AARCH64_ABS64
#define R_AARCH64_ABS64      257
#endif /* R_AARCH64_ABS64 */
#ifndef R_AARCH64_ABS32
#define R_AARCH64_ABS32      258
#endif /* R_AARCH64_ABS32 */
#ifndef R_AARCH64_ABS16
#define R_AARCH64_ABS16      259
#endif /* R_AARCH64_ABS16 */
#ifndef R_AARCH64_PREL64
#define R_AARCH64_PREL64     260
#endif /* R_AARCH64_PREL64 */
#ifndef R_AARCH64_PREL32
#define R_AARCH64_PREL32     261
#endif /* R_AARCH64_PREL32 */
#ifndef R_AARCH64_PREL16
#define R_AARCH64_PREL16     262
#endif /* R_AARCH64_PREL16 */
#ifndef R_AARCH64_MOVW_UABS_G0
#define R_AARCH64_MOVW_UABS_G0 263
#endif /* R_AARCH64_MOVW_UABS_G0 */
#ifndef R_AARCH64_MOVW_UABS_G0_NC
#define R_AARCH64_MOVW_UABS_G0_NC 264
#endif /* R_AARCH64_MOVW_UABS_G0_NC */
#ifndef R_AARCH64_MOVW_UABS_G1
#define R_AARCH64_MOVW_UABS_G1 265
#endif /* R_AARCH64_MOVW_UABS_G1 */
#ifndef R_AARCH64_MOVW_UABS_G1_NC
#define R_AARCH64_MOVW_UABS_G1_NC 266
#endif /* R_AARCH64_MOVW_UABS_G1_NC */
#ifndef R_AARCH64_MOVW_UABS_G2
#define R_AARCH64_MOVW_UABS_G2 267
#endif /* R_AARCH64_MOVW_UABS_G2 */
#ifndef R_AARCH64_MOVW_UABS_G2_NC
#define R_AARCH64_MOVW_UABS_G2_NC 268
#endif /* R_AARCH64_MOVW_UABS_G2_NC */
#ifndef R_AARCH64_MOVW_UABS_G3
#define R_AARCH64_MOVW_UABS_G3 269
#endif /* R_AARCH64_MOVW_UABS_G3 */
#ifndef R_AARCH64_MOVW_SABS_G0
#define R_AARCH64_MOVW_SABS_G0 270
#endif /* R_AARCH64_MOVW_SABS_G0 */
#ifndef R_AARCH64_MOVW_SABS_G1
#define R_AARCH64_MOVW_SABS_G1 271
#endif /* R_AARCH64_MOVW_SABS_G1 */
#ifndef R_AARCH64_MOVW_SABS_G2
#define R_AARCH64_MOVW_SABS_G2 272
#endif /* R_AARCH64_MOVW_SABS_G2 */
#ifndef R_AARCH64_LD_PREL_LO19
#define R_AARCH64_LD_PREL_LO19 273
#endif /* R_AARCH64_LD_PREL_LO19 */
#ifndef R_AARCH64_ADR_PREL_LO21
#define R_AARCH64_ADR_PREL_LO21 274
#endif /* R_AARCH64_ADR_PREL_LO21 */
#ifndef R_AARCH64_ADR_PREL_PG_HI21
#define R_AARCH64_ADR_PREL_PG_HI21 275
#endif /* R_AARCH64_ADR_PREL_PG_HI21 */
#ifndef R_AARCH64_ADR_PREL_PG_HI21_NC
#define R_AARCH64_ADR_PREL_PG_HI21_NC 276
#endif /* R_AARCH64_ADR_PREL_PG_HI21_NC */
#ifndef R_AARCH64_ADD_ABS_LO12_NC
#define R_AARCH64_ADD_ABS_LO12_NC 277
#endif /* R_AARCH64_ADD_ABS_LO12_NC */
#ifndef R_AARCH64_LDST8_ABS_LO12_NC
#define R_AARCH64_LDST8_ABS_LO12_NC 278
#endif /* R_AARCH64_LDST8_ABS_LO12_NC */
#ifndef R_AARCH64_TSTBR14
#define R_AARCH64_TSTBR14    279
#endif /* R_AARCH64_TSTBR14 */
#ifndef R_AARCH64_CONDBR19
#define R_AARCH64_CONDBR19   280
#endif /* R_AARCH64_CONDBR19 */
#ifndef R_AARCH64_JUMP26
#define R_AARCH64_JUMP26     282
#endif /* R_AARCH64_JUMP26 */
#ifndef R_AARCH64_CALL26
#define R_AARCH64_CALL26     283
#endif /* R_AARCH64_CALL26 */
#ifndef R_AARCH64_LDST16_ABS_LO12_NC
#define R_AARCH64_LDST16_ABS_LO12_NC 284
#endif /* R_AARCH64_LDST16_ABS_LO12_NC */
#ifndef R_AARCH64_LDST32_ABS_LO12_NC
#define R_AARCH64_LDST32_ABS_LO12_NC 285
#endif /* R_AARCH64_LDST32_ABS_LO12_NC */
#ifndef R_AARCH64_LDST64_ABS_LO12_NC
#define R_AARCH64_LDST64_ABS_LO12_NC 286
#endif /* R_AARCH64_LDST64_ABS_LO12_NC */
#ifndef R_AARCH64_MOVW_PREL_G0
#define R_AARCH64_MOVW_PREL_G0 287
#endif /* R_AARCH64_MOVW_PREL_G0 */
#ifndef R_AARCH64_MOVW_PREL_G0_NC
#define R_AARCH64_MOVW_PREL_G0_NC 288
#endif /* R_AARCH64_MOVW_PREL_G0_NC */
#ifndef R_AARCH64_MOVW_PREL_G1
#define R_AARCH64_MOVW_PREL_G1 289
#endif /* R_AARCH64_MOVW_PREL_G1 */
#ifndef R_AARCH64_MOVW_PREL_G1_NC
#define R_AARCH64_MOVW_PREL_G1_NC 290
#endif /* R_AARCH64_MOVW_PREL_G1_NC */
#ifndef R_AARCH64_MOVW_PREL_G2
#define R_AARCH64_MOVW_PREL_G2 291
#endif /* R_AARCH64_MOVW_PREL_G2 */
#ifndef R_AARCH64_MOVW_PREL_G2_NC
#define R_AARCH64_MOVW_PREL_G2_NC 292
#endif /* R_AARCH64_MOVW_PREL_G2_NC */
#ifndef R_AARCH64_MOVW_PREL_G3
#define R_AARCH64_MOVW_PREL_G3 293
#endif /* R_AARCH64_MOVW_PREL_G3 */
#ifndef R_AARCH64_LDST128_ABS_LO12_NC
#define R_AARCH64_LDST128_ABS_LO12_NC 299
#endif /* R_AARCH64_LDST128_ABS_LO12_NC */
#ifndef R_AARCH64_MOVW_GOTOFF_G0
#define R_AARCH64_MOVW_GOTOFF_G0 300
#endif /* R_AARCH64_MOVW_GOTOFF_G0 */
#ifndef R_AARCH64_MOVW_GOTOFF_G0_NC
#define R_AARCH64_MOVW_GOTOFF_G0_NC 301
#endif /* R_AARCH64_MOVW_GOTOFF_G0_NC */
#ifndef R_AARCH64_MOVW_GOTOFF_G1
#define R_AARCH64_MOVW_GOTOFF_G1 302
#endif /* R_AARCH64_MOVW_GOTOFF_G1 */
#ifndef R_AARCH64_MOVW_GOTOFF_G1_NC
#define R_AARCH64_MOVW_GOTOFF_G1_NC 303
#endif /* R_AARCH64_MOVW_GOTOFF_G1_NC */
#ifndef R_AARCH64_MOVW_GOTOFF_G2
#define R_AARCH64_MOVW_GOTOFF_G2 304
#endif /* R_AARCH64_MOVW_GOTOFF_G2 */
#ifndef R_AARCH64_MOVW_GOTOFF_G2_NC
#define R_AARCH64_MOVW_GOTOFF_G2_NC 305
#endif /* R_AARCH64_MOVW_GOTOFF_G2_NC */
#ifndef R_AARCH64_MOVW_GOTOFF_G3
#define R_AARCH64_MOVW_GOTOFF_G3 306
#endif /* R_AARCH64_MOVW_GOTOFF_G3 */
#ifndef R_AARCH64_GOTREL64
#define R_AARCH64_GOTREL64   307
#endif /* R_AARCH64_GOTREL64 */
#ifndef R_AARCH64_GOTREL32
#define R_AARCH64_GOTREL32   308
#endif /* R_AARCH64_GOTREL32 */
#ifndef R_AARCH64_GOT_LD_PREL19
#define R_AARCH64_GOT_LD_PREL19 309
#endif /* R_AARCH64_GOT_LD_PREL19 */
#ifndef R_AARCH64_LD64_GOTOFF_LO15
#define R_AARCH64_LD64_GOTOFF_LO15 310
#endif /* R_AARCH64_LD64_GOTOFF_LO15 */
#ifndef R_AARCH64_ADR_GOT_PAGE
#define R_AARCH64_ADR_GOT_PAGE 311
#endif /* R_AARCH64_ADR_GOT_PAGE */
#ifndef R_AARCH64_LD64_GOT_LO12_NC
#define R_AARCH64_LD64_GOT_LO12_NC 312
#endif /* R_AARCH64_LD64_GOT_LO12_NC */
#ifndef R_AARCH64_LD64_GOTPAGE_LO15
#define R_AARCH64_LD64_GOTPAGE_LO15 313
#endif /* R_AARCH64_LD64_GOTPAGE_LO15 */
#ifndef R_AARCH64_TLSGD_ADR_PREL21
#define R_AARCH64_TLSGD_ADR_PREL21 512
#endif /* R_AARCH64_TLSGD_ADR_PREL21 */
#ifndef R_AARCH64_TLSGD_ADR_PAGE21
#define R_AARCH64_TLSGD_ADR_PAGE21 513
#endif /* R_AARCH64_TLSGD_ADR_PAGE21 */
#ifndef R_AARCH64_TLSGD_ADD_LO12_NC
#define R_AARCH64_TLSGD_ADD_LO12_NC 514
#endif /* R_AARCH64_TLSGD_ADD_LO12_NC */
#ifndef R_AARCH64_TLSGD_MOVW_G1
#define R_AARCH64_TLSGD_MOVW_G1 515
#endif /* R_AARCH64_TLSGD_MOVW_G1 */
#ifndef R_AARCH64_TLSGD_MOVW_G0_NC
#define R_AARCH64_TLSGD_MOVW_G0_NC 516
#endif /* R_AARCH64_TLSGD_MOVW_G0_NC */
#ifndef R_AARCH64_TLSLD_ADR_PREL21
#define R_AARCH64_TLSLD_ADR_PREL21 517
#endif /* R_AARCH64_TLSLD_ADR_PREL21 */
#ifndef R_AARCH64_TLSLD_ADR_PAGE21
#define R_AARCH64_TLSLD_ADR_PAGE21 518
#endif /* R_AARCH64_TLSLD_ADR_PAGE21 */
#ifndef R_AARCH64_TLSLD_ADD_LO12_NC
#define R_AARCH64_TLSLD_ADD_LO12_NC 519
#endif /* R_AARCH64_TLSLD_ADD_LO12_NC */
#ifndef R_AARCH64_TLSLD_MOVW_G1
#define R_AARCH64_TLSLD_MOVW_G1 520
#endif /* R_AARCH64_TLSLD_MOVW_G1 */
#ifndef R_AARCH64_TLSLD_MOVW_G0_NC
#define R_AARCH64_TLSLD_MOVW_G0_NC 521
#endif /* R_AARCH64_TLSLD_MOVW_G0_NC */
#ifndef R_AARCH64_TLSLD_LD_PREL19
#define R_AARCH64_TLSLD_LD_PREL19 522
#endif /* R_AARCH64_TLSLD_LD_PREL19 */
#ifndef R_AARCH64_TLSLD_MOVW_DTPREL_G2
#define R_AARCH64_TLSLD_MOVW_DTPREL_G2 523
#endif /* R_AARCH64_TLSLD_MOVW_DTPREL_G2 */
#ifndef R_AARCH64_TLSLD_MOVW_DTPREL_G1
#define R_AARCH64_TLSLD_MOVW_DTPREL_G1 524
#endif /* R_AARCH64_TLSLD_MOVW_DTPREL_G1 */
#ifndef R_AARCH64_TLSLD_MOVW_DTPREL_G1_NC
#define R_AARCH64_TLSLD_MOVW_DTPREL_G1_NC 525
#endif /* R_AARCH64_TLSLD_MOVW_DTPREL_G1_NC */
#ifndef R_AARCH64_TLSLD_MOVW_DTPREL_G0
#define R_AARCH64_TLSLD_MOVW_DTPREL_G0 526
#endif /* R_AARCH64_TLSLD_MOVW_DTPREL_G0 */
#ifndef R_AARCH64_TLSLD_MOVW_DTPREL_G0_NC
#define R_AARCH64_TLSLD_MOVW_DTPREL_G0_NC 527
#endif /* R_AARCH64_TLSLD_MOVW_DTPREL_G0_NC */
#ifndef R_AARCH64_TLSLD_ADD_DTPREL_HI12
#define R_AARCH64_TLSLD_ADD_DTPREL_HI12 528
#endif /* R_AARCH64_TLSLD_ADD_DTPREL_HI12 */
#ifndef R_AARCH64_TLSLD_ADD_DTPREL_LO12
#define R_AARCH64_TLSLD_ADD_DTPREL_LO12 529
#endif /* R_AARCH64_TLSLD_ADD_DTPREL_LO12 */
#ifndef R_AARCH64_TLSLD_ADD_DTPREL_LO12_NC
#define R_AARCH64_TLSLD_ADD_DTPREL_LO12_NC 530
#endif /* R_AARCH64_TLSLD_ADD_DTPREL_LO12_NC */
#ifndef R_AARCH64_TLSLD_LDST8_DTPREL_LO12
#define R_AARCH64_TLSLD_LDST8_DTPREL_LO12 531
#endif /* R_AARCH64_TLSLD_LDST8_DTPREL_LO12 */
#ifndef R_AARCH64_TLSLD_LDST8_DTPREL_LO12_NC
#define R_AARCH64_TLSLD_LDST8_DTPREL_LO12_NC 532
#endif /* R_AARCH64_TLSLD_LDST8_DTPREL_LO12_NC */
#ifndef R_AARCH64_TLSLD_LDST16_DTPREL_LO12
#define R_AARCH64_TLSLD_LDST16_DTPREL_LO12 533
#endif /* R_AARCH64_TLSLD_LDST16_DTPREL_LO12 */
#ifndef R_AARCH64_TLSLD_LDST16_DTPREL_LO12_NC
#define R_AARCH64_TLSLD_LDST16_DTPREL_LO12_NC 534
#endif /* R_AARCH64_TLSLD_LDST16_DTPREL_LO12_NC */
#ifndef R_AARCH64_TLSLD_LDST32_DTPREL_LO12
#define R_AARCH64_TLSLD_LDST32_DTPREL_LO12 535
#endif /* R_AARCH64_TLSLD_LDST32_DTPREL_LO12 */
#ifndef R_AARCH64_TLSLD_LDST32_DTPREL_LO12_NC
#define R_AARCH64_TLSLD_LDST32_DTPREL_LO12_NC 536
#endif /* R_AARCH64_TLSLD_LDST32_DTPREL_LO12_NC */
#ifndef R_AARCH64_TLSLD_LDST64_DTPREL_LO12
#define R_AARCH64_TLSLD_LDST64_DTPREL_LO12 537
#endif /* R_AARCH64_TLSLD_LDST64_DTPREL_LO12 */
#ifndef R_AARCH64_TLSLD_LDST64_DTPREL_LO12_NC
#define R_AARCH64_TLSLD_LDST64_DTPREL_LO12_NC 538
#endif /* R_AARCH64_TLSLD_LDST64_DTPREL_LO12_NC */
#ifndef R_AARCH64_TLSIE_MOVW_GOTTPREL_G1
#define R_AARCH64_TLSIE_MOVW_GOTTPREL_G1 539
#endif /* R_AARCH64_TLSIE_MOVW_GOTTPREL_G1 */
#ifndef R_AARCH64_TLSIE_MOVW_GOTTPREL_G0_NC
#define R_AARCH64_TLSIE_MOVW_GOTTPREL_G0_NC 540
#endif /* R_AARCH64_TLSIE_MOVW_GOTTPREL_G0_NC */
#ifndef R_AARCH64_TLSIE_ADR_GOTTPREL_PAGE21
#define R_AARCH64_TLSIE_ADR_GOTTPREL_PAGE21 541
#endif /* R_AARCH64_TLSIE_ADR_GOTTPREL_PAGE21 */
#ifndef R_AARCH64_TLSIE_LD64_GOTTPREL_LO12_NC
#define R_AARCH64_TLSIE_LD64_GOTTPREL_LO12_NC 542
#endif /* R_AARCH64_TLSIE_LD64_GOTTPREL_LO12_NC */
#ifndef R_AARCH64_TLSIE_LD_GOTTPREL_PREL19
#define R_AARCH64_TLSIE_LD_GOTTPREL_PREL19 543
#endif /* R_AARCH64_TLSIE_LD_GOTTPREL_PREL19 */
#ifndef R_AARCH64_TLSLE_MOVW_TPREL_G2
#define R_AARCH64_TLSLE_MOVW_TPREL_G2 544
#endif /* R_AARCH64_TLSLE_MOVW_TPREL_G2 */
#ifndef R_AARCH64_TLSLE_MOVW_TPREL_G1
#define R_AARCH64_TLSLE_MOVW_TPREL_G1 545
#endif /* R_AARCH64_TLSLE_MOVW_TPREL_G1 */
#ifndef R_AARCH64_TLSLE_MOVW_TPREL_G1_NC
#define R_AARCH64_TLSLE_MOVW_TPREL_G1_NC 546
#endif /* R_AARCH64_TLSLE_MOVW_TPREL_G1_NC */
#ifndef R_AARCH64_TLSLE_MOVW_TPREL_G0
#define R_AARCH64_TLSLE_MOVW_TPREL_G0 547
#endif /* R_AARCH64_TLSLE_MOVW_TPREL_G0 */
#ifndef R_AARCH64_TLSLE_MOVW_TPREL_G0_NC
#define R_AARCH64_TLSLE_MOVW_TPREL_G0_NC 548
#endif /* R_AARCH64_TLSLE_MOVW_TPREL_G0_NC */
#ifndef R_AARCH64_TLSLE_ADD_TPREL_HI12
#define R_AARCH64_TLSLE_ADD_TPREL_HI12 549
#endif /* R_AARCH64_TLSLE_ADD_TPREL_HI12 */
#ifndef R_AARCH64_TLSLE_ADD_TPREL_LO12
#define R_AARCH64_TLSLE_ADD_TPREL_LO12 550
#endif /* R_AARCH64_TLSLE_ADD_TPREL_LO12 */
#ifndef R_AARCH64_TLSLE_ADD_TPREL_LO12_NC
#define R_AARCH64_TLSLE_ADD_TPREL_LO12_NC 551
#endif /* R_AARCH64_TLSLE_ADD_TPREL_LO12_NC */
#ifndef R_AARCH64_TLSLE_LDST8_TPREL_LO12
#define R_AARCH64_TLSLE_LDST8_TPREL_LO12 552
#endif /* R_AARCH64_TLSLE_LDST8_TPREL_LO12 */
#ifndef R_AARCH64_TLSLE_LDST8_TPREL_LO12_NC
#define R_AARCH64_TLSLE_LDST8_TPREL_LO12_NC 553
#endif /* R_AARCH64_TLSLE_LDST8_TPREL_LO12_NC */
#ifndef R_AARCH64_TLSLE_LDST16_TPREL_LO12
#define R_AARCH64_TLSLE_LDST16_TPREL_LO12 554
#endif /* R_AARCH64_TLSLE_LDST16_TPREL_LO12 */
#ifndef R_AARCH64_TLSLE_LDST16_TPREL_LO12_NC
#define R_AARCH64_TLSLE_LDST16_TPREL_LO12_NC 555
#endif /* R_AARCH64_TLSLE_LDST16_TPREL_LO12_NC */
#ifndef R_AARCH64_TLSLE_LDST32_TPREL_LO12
#define R_AARCH64_TLSLE_LDST32_TPREL_LO12 556
#endif /* R_AARCH64_TLSLE_LDST32_TPREL_LO12 */
#ifndef R_AARCH64_TLSLE_LDST32_TPREL_LO12_NC
#define R_AARCH64_TLSLE_LDST32_TPREL_LO12_NC 557
#endif /* R_AARCH64_TLSLE_LDST32_TPREL_LO12_NC */
#ifndef R_AARCH64_TLSLE_LDST64_TPREL_LO12
#define R_AARCH64_TLSLE_LDST64_TPREL_LO12 558
#endif /* R_AARCH64_TLSLE_LDST64_TPREL_LO12 */
#ifndef R_AARCH64_TLSLE_LDST64_TPREL_LO12_NC
#define R_AARCH64_TLSLE_LDST64_TPREL_LO12_NC 559
#endif /* R_AARCH64_TLSLE_LDST64_TPREL_LO12_NC */
#ifndef R_AARCH64_TLSDESC_LD_PREL19
#define R_AARCH64_TLSDESC_LD_PREL19 560
#endif /* R_AARCH64_TLSDESC_LD_PREL19 */
#ifndef R_AARCH64_TLSDESC_ADR_PREL21
#define R_AARCH64_TLSDESC_ADR_PREL21 561
#endif /* R_AARCH64_TLSDESC_ADR_PREL21 */
#ifndef R_AARCH64_TLSDESC_ADR_PAGE21
#define R_AARCH64_TLSDESC_ADR_PAGE21 562
#endif /* R_AARCH64_TLSDESC_ADR_PAGE21 */
#ifndef R_AARCH64_TLSDESC_LD64_LO12
#define R_AARCH64_TLSDESC_LD64_LO12 563
#endif /* R_AARCH64_TLSDESC_LD64_LO12 */
#ifndef R_AARCH64_TLSDESC_ADD_LO12
#define R_AARCH64_TLSDESC_ADD_LO12 564
#endif /* R_AARCH64_TLSDESC_ADD_LO12 */
#ifndef R_AARCH64_TLSDESC_OFF_G1
#define R_AARCH64_TLSDESC_OFF_G1 565
#endif /* R_AARCH64_TLSDESC_OFF_G1 */
#ifndef R_AARCH64_TLSDESC_OFF_G0_NC
#define R_AARCH64_TLSDESC_OFF_G0_NC 566
#endif /* R_AARCH64_TLSDESC_OFF_G0_NC */
#ifndef R_AARCH64_TLSDESC_LDR
#define R_AARCH64_TLSDESC_LDR 567
#endif /* R_AARCH64_TLSDESC_LDR */
#ifndef R_AARCH64_TLSDESC_ADD
#define R_AARCH64_TLSDESC_ADD 568
#endif /* R_AARCH64_TLSDESC_ADD */
#ifndef R_AARCH64_TLSDESC_CALL
#define R_AARCH64_TLSDESC_CALL 569
#endif /* R_AARCH64_TLSDESC_CALL */
#ifndef R_AARCH64_TLSLE_LDST128_TPREL_LO12
#define R_AARCH64_TLSLE_LDST128_TPREL_LO12 570
#endif /* R_AARCH64_TLSLE_LDST128_TPREL_LO12 */
#ifndef R_AARCH64_TLSLE_LDST128_TPREL_LO12_NC
#define R_AARCH64_TLSLE_LDST128_TPREL_LO12_NC 571
#endif /* R_AARCH64_TLSLE_LDST128_TPREL_LO12_NC */
#ifndef R_AARCH64_TLSLD_LDST128_DTPREL_LO12
#define R_AARCH64_TLSLD_LDST128_DTPREL_LO12 572
#endif /* R_AARCH64_TLSLD_LDST128_DTPREL_LO12 */
#ifndef R_AARCH64_TLSLD_LDST128_DTPREL_LO12_NC
#define R_AARCH64_TLSLD_LDST128_DTPREL_LO12_NC 573
#endif /* R_AARCH64_TLSLD_LDST128_DTPREL_LO12_NC */
#ifndef R_AARCH64_COPY
#define R_AARCH64_COPY       1024
#endif /* R_AARCH64_COPY */
#ifndef R_AARCH64_GLOB_DAT
#define R_AARCH64_GLOB_DAT   1025
#endif /* R_AARCH64_GLOB_DAT */
#ifndef R_AARCH64_JUMP_SLOT
#define R_AARCH64_JUMP_SLOT  1026
#endif /* R_AARCH64_JUMP_SLOT */
#ifndef R_AARCH64_RELATIVE
#define R_AARCH64_RELATIVE   1027
#endif /* R_AARCH64_RELATIVE */
#ifndef R_AARCH64_TLS_DTPMOD
#define R_AARCH64_TLS_DTPMOD 1028
#endif /* R_AARCH64_TLS_DTPMOD */
#ifndef R_AARCH64_TLS_DTPREL
#define R_AARCH64_TLS_DTPREL 1029
#endif /* R_AARCH64_TLS_DTPREL */
#ifndef R_AARCH64_TLS_TPREL
#define R_AARCH64_TLS_TPREL  1030
#endif /* R_AARCH64_TLS_TPREL */
#ifndef R_AARCH64_TLSDESC
#define R_AARCH64_TLSDESC    1031
#endif /* R_AARCH64_TLSDESC */
#ifndef R_AARCH64_IRELATIVE
#define R_AARCH64_IRELATIVE  1032
#endif /* R_AARCH64_IRELATIVE */
