/* Created by build_access.py */
/* returns string of length 0 if invalid arg */
const char * dwarf_get_elf_relocname_sparc(unsigned long);
#ifndef R_SPARC_NONE
#define R_SPARC_NONE         0
#endif /* R_SPARC_NONE */
#ifndef R_SPARC_8
#define R_SPARC_8            1
#endif /* R_SPARC_8 */
#ifndef R_SPARC_16
#define R_SPARC_16           2
#endif /* R_SPARC_16 */
#ifndef R_SPARC_32
#define R_SPARC_32           3
#endif /* R_SPARC_32 */
#ifndef R_SPARC_DISP8
#define R_SPARC_DISP8        4
#endif /* R_SPARC_DISP8 */
#ifndef R_SPARC_DISP16
#define R_SPARC_DISP16       5
#endif /* R_SPARC_DISP16 */
#ifndef R_SPARC_DISP32
#define R_SPARC_DISP32       6
#endif /* R_SPARC_DISP32 */
#ifndef R_SPARC_WDISP30
#define R_SPARC_WDISP30      7
#endif /* R_SPARC_WDISP30 */
#ifndef R_SPARC_WDISP22
#define R_SPARC_WDISP22      8
#endif /* R_SPARC_WDISP22 */
#ifndef R_SPARC_HI22
#define R_SPARC_HI22         9
#endif /* R_SPARC_HI22 */
#ifndef R_SPARC_22
#define R_SPARC_22           10
#endif /* R_SPARC_22 */
#ifndef R_SPARC_13
#define R_SPARC_13           11
#endif /* R_SPARC_13 */
#ifndef R_SPARC_LO10
#define R_SPARC_LO10         12
#endif /* R_SPARC_LO10 */
#ifndef R_SPARC_GOT10
#define R_SPARC_GOT10        13
#endif /* R_SPARC_GOT10 */
#ifndef R_SPARC_GOT13
#define R_SPARC_GOT13        14
#endif /* R_SPARC_GOT13 */
#ifndef R_SPARC_GOT22
#define R_SPARC_GOT22        15
#endif /* R_SPARC_GOT22 */
#ifndef R_SPARC_PC10
#define R_SPARC_PC10         16
#endif /* R_SPARC_PC10 */
#ifndef R_SPARC_PC22
#define R_SPARC_PC22         17
#endif /* R_SPARC_PC22 */
#ifndef R_SPARC_WPLT30
#define R_SPARC_WPLT30       18
#endif /* R_SPARC_WPLT30 */
#ifndef R_SPARC_COPY
#define R_SPARC_COPY         19
#endif /* R_SPARC_COPY */
#ifndef R_SPARC_GLOB_DAT
#define R_SPARC_GLOB_DAT     20
#endif /* R_SPARC_GLOB_DAT */
#ifndef R_SPARC_JMP_SLOT
#define R_SPARC_JMP_SLOT     21
#endif /* R_SPARC_JMP_SLOT */
#ifndef R_SPARC_RELATIVE
#define R_SPARC_RELATIVE     22
#endif /* R_SPARC_RELATIVE */
#ifndef R_SPARC_UA32
#define R_SPARC_UA32         23
#endif /* R_SPARC_UA32 */
#ifndef R_SPARC_PLT32
#define R_SPARC_PLT32        24
#endif /* R_SPARC_PLT32 */
#ifndef R_SPARC_HIPLT22
#define R_SPARC_HIPLT22      25
#endif /* R_SPARC_HIPLT22 */
#ifndef R_SPARC_LOPLT10
#define R_SPARC_LOPLT10      26
#endif /* R_SPARC_LOPLT10 */
#ifndef R_SPARC_PCPLT32
#define R_SPARC_PCPLT32      27
#endif /* R_SPARC_PCPLT32 */
#ifndef R_SPARC_PCPLT22
#define R_SPARC_PCPLT22      28
#endif /* R_SPARC_PCPLT22 */
#ifndef R_SPARC_PCPLT10
#define R_SPARC_PCPLT10      29
#endif /* R_SPARC_PCPLT10 */
#ifndef R_SPARC_10
#define R_SPARC_10           30
#endif /* R_SPARC_10 */
#ifndef R_SPARC_11
#define R_SPARC_11           31
#endif /* R_SPARC_11 */
#ifndef R_SPARC_64
#define R_SPARC_64           32
#endif /* R_SPARC_64 */
#ifndef R_SPARC_OLO10
#define R_SPARC_OLO10        33
#endif /* R_SPARC_OLO10 */
#ifndef R_SPARC_HH22
#define R_SPARC_HH22         34
#endif /* R_SPARC_HH22 */
#ifndef R_SPARC_HM10
#define R_SPARC_HM10         35
#endif /* R_SPARC_HM10 */
#ifndef R_SPARC_LM22
#define R_SPARC_LM22         36
#endif /* R_SPARC_LM22 */
#ifndef R_SPARC_PC_HH22
#define R_SPARC_PC_HH22      37
#endif /* R_SPARC_PC_HH22 */
#ifndef R_SPARC_PC_HM10
#define R_SPARC_PC_HM10      38
#endif /* R_SPARC_PC_HM10 */
#ifndef R_SPARC_PC_LM22
#define R_SPARC_PC_LM22      39
#endif /* R_SPARC_PC_LM22 */
#ifndef R_SPARC_WDISP16
#define R_SPARC_WDISP16      40
#endif /* R_SPARC_WDISP16 */
#ifndef R_SPARC_WDISP19
#define R_SPARC_WDISP19      41
#endif /* R_SPARC_WDISP19 */
#ifndef R_SPARC_GLOB_JMP
#define R_SPARC_GLOB_JMP     42
#endif /* R_SPARC_GLOB_JMP */
#ifndef R_SPARC_7
#define R_SPARC_7            43
#endif /* R_SPARC_7 */
#ifndef R_SPARC_5
#define R_SPARC_5            44
#endif /* R_SPARC_5 */
#ifndef R_SPARC_6
#define R_SPARC_6            45
#endif /* R_SPARC_6 */
#ifndef R_SPARC_DISP64
#define R_SPARC_DISP64       46
#endif /* R_SPARC_DISP64 */
#ifndef R_SPARC_PLT64
#define R_SPARC_PLT64        47
#endif /* R_SPARC_PLT64 */
#ifndef R_SPARC_HIX22
#define R_SPARC_HIX22        48
#endif /* R_SPARC_HIX22 */
#ifndef R_SPARC_LOX10
#define R_SPARC_LOX10        49
#endif /* R_SPARC_LOX10 */
#ifndef R_SPARC_H44
#define R_SPARC_H44          50
#endif /* R_SPARC_H44 */
#ifndef R_SPARC_M44
#define R_SPARC_M44          51
#endif /* R_SPARC_M44 */
#ifndef R_SPARC_L44
#define R_SPARC_L44          52
#endif /* R_SPARC_L44 */
#ifndef R_SPARC_REGISTER
#define R_SPARC_REGISTER     53
#endif /* R_SPARC_REGISTER */
#ifndef R_SPARC_UA64
#define R_SPARC_UA64         54
#endif /* R_SPARC_UA64 */
#ifndef R_SPARC_UA16
#define R_SPARC_UA16         55
#endif /* R_SPARC_UA16 */
#ifndef R_SPARC_TLS_GD_HI22
#define R_SPARC_TLS_GD_HI22  56
#endif /* R_SPARC_TLS_GD_HI22 */
#ifndef R_SPARC_TLS_GD_LO10
#define R_SPARC_TLS_GD_LO10  57
#endif /* R_SPARC_TLS_GD_LO10 */
#ifndef R_SPARC_TLS_GD_ADD
#define R_SPARC_TLS_GD_ADD   58
#endif /* R_SPARC_TLS_GD_ADD */
#ifndef R_SPARC_TLS_GD_CALL
#define R_SPARC_TLS_GD_CALL  59
#endif /* R_SPARC_TLS_GD_CALL */
#ifndef R_SPARC_TLS_LDM_HI22
#define R_SPARC_TLS_LDM_HI22 60
#endif /* R_SPARC_TLS_LDM_HI22 */
#ifndef R_SPARC_TLS_LDM_LO10
#define R_SPARC_TLS_LDM_LO10 61
#endif /* R_SPARC_TLS_LDM_LO10 */
#ifndef R_SPARC_TLS_LDM_ADD
#define R_SPARC_TLS_LDM_ADD  62
#endif /* R_SPARC_TLS_LDM_ADD */
#ifndef R_SPARC_TLS_LDM_CALL
#define R_SPARC_TLS_LDM_CALL 63
#endif /* R_SPARC_TLS_LDM_CALL */
#ifndef R_SPARC_TLS_LDO_HIX22
#define R_SPARC_TLS_LDO_HIX22 64
#endif /* R_SPARC_TLS_LDO_HIX22 */
#ifndef R_SPARC_TLS_LDO_LOX10
#define R_SPARC_TLS_LDO_LOX10 65
#endif /* R_SPARC_TLS_LDO_LOX10 */
#ifndef R_SPARC_TLS_LDO_ADD
#define R_SPARC_TLS_LDO_ADD  66
#endif /* R_SPARC_TLS_LDO_ADD */
#ifndef R_SPARC_TLS_IE_HI22
#define R_SPARC_TLS_IE_HI22  67
#endif /* R_SPARC_TLS_IE_HI22 */
#ifndef R_SPARC_TLS_IE_LO10
#define R_SPARC_TLS_IE_LO10  68
#endif /* R_SPARC_TLS_IE_LO10 */
#ifndef R_SPARC_TLS_IE_LD
#define R_SPARC_TLS_IE_LD    69
#endif /* R_SPARC_TLS_IE_LD */
#ifndef R_SPARC_TLS_IE_LDX
#define R_SPARC_TLS_IE_LDX   70
#endif /* R_SPARC_TLS_IE_LDX */
#ifndef R_SPARC_TLS_IE_ADD
#define R_SPARC_TLS_IE_ADD   71
#endif /* R_SPARC_TLS_IE_ADD */
#ifndef R_SPARC_TLS_LE_HIX22
#define R_SPARC_TLS_LE_HIX22 72
#endif /* R_SPARC_TLS_LE_HIX22 */
#ifndef R_SPARC_TLS_LE_LOX10
#define R_SPARC_TLS_LE_LOX10 73
#endif /* R_SPARC_TLS_LE_LOX10 */
#ifndef R_SPARC_TLS_DTPMOD32
#define R_SPARC_TLS_DTPMOD32 74
#endif /* R_SPARC_TLS_DTPMOD32 */
#ifndef R_SPARC_TLS_DTPMOD64
#define R_SPARC_TLS_DTPMOD64 75
#endif /* R_SPARC_TLS_DTPMOD64 */
#ifndef R_SPARC_TLS_DTPOFF32
#define R_SPARC_TLS_DTPOFF32 76
#endif /* R_SPARC_TLS_DTPOFF32 */
#ifndef R_SPARC_TLS_DTPOFF64
#define R_SPARC_TLS_DTPOFF64 77
#endif /* R_SPARC_TLS_DTPOFF64 */
#ifndef R_SPARC_TLS_TPOFF32
#define R_SPARC_TLS_TPOFF32  78
#endif /* R_SPARC_TLS_TPOFF32 */
#ifndef R_SPARC_TLS_TPOFF64
#define R_SPARC_TLS_TPOFF64  79
#endif /* R_SPARC_TLS_TPOFF64 */
#ifndef R_SPARC_GOTDATA_HIX22
#define R_SPARC_GOTDATA_HIX22 80
#endif /* R_SPARC_GOTDATA_HIX22 */
#ifndef R_SPARC_GOTDATA_LOX10
#define R_SPARC_GOTDATA_LOX10 81
#endif /* R_SPARC_GOTDATA_LOX10 */
#ifndef R_SPARC_GOTDATA_OP_HIX22
#define R_SPARC_GOTDATA_OP_HIX22 82
#endif /* R_SPARC_GOTDATA_OP_HIX22 */
#ifndef R_SPARC_GOTDATA_OP_LOX10
#define R_SPARC_GOTDATA_OP_LOX10 83
#endif /* R_SPARC_GOTDATA_OP_LOX10 */
#ifndef R_SPARC_GOTDATA_OP
#define R_SPARC_GOTDATA_OP   84
#endif /* R_SPARC_GOTDATA_OP */
#ifndef R_SPARC_H34
#define R_SPARC_H34          85
#endif /* R_SPARC_H34 */
#ifndef R_SPARC_SIZE32
#define R_SPARC_SIZE32       86
#endif /* R_SPARC_SIZE32 */
#ifndef R_SPARC_SIZE64
#define R_SPARC_SIZE64       87
#endif /* R_SPARC_SIZE64 */
#ifndef R_SPARC_WDISP10
#define R_SPARC_WDISP10      88
#endif /* R_SPARC_WDISP10 */
#ifndef R_SPARC_JMP_IREL
#define R_SPARC_JMP_IREL     248
#endif /* R_SPARC_JMP_IREL */
#ifndef R_SPARC_IRELATIVE
#define R_SPARC_IRELATIVE    249
#endif /* R_SPARC_IRELATIVE */
#ifndef R_SPARC_GNU_VTINHERIT
#define R_SPARC_GNU_VTINHERIT 250
#endif /* R_SPARC_GNU_VTINHERIT */
#ifndef R_SPARC_GNU_VTENTRY
#define R_SPARC_GNU_VTENTRY  251
#endif /* R_SPARC_GNU_VTENTRY */
#ifndef R_SPARC_REV32
#define R_SPARC_REV32        252
#endif /* R_SPARC_REV32 */
